%%%%%%%%%%%%%% GENERAL ABM TOY MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%
stream=RandStream.create('mrg32k3a','Seed',13);
RandStream.setGlobalStream(stream);
stream.Substream=3;
repeatstate1=stream.State;

mrun=1;
stream.Substream=mrun;

%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
%@@@@@@@@@@@@@@@@@@@@@    PARAMETERS    @@@@@@@@@@@@@@@@@@@@@@@@@
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

TMAX=30;
TSTART=10;

NLENGTH=100;
NWIDTH=100;

globalind=reshape((1:NLENGTH*NWIDTH),NLENGTH,NWIDTH);


%%%%%%%%%%%%%% Global Constraining Factors %%%%%%%%%%%%%%%%

MKTACC=0.2;
MKTINF= 0.2;
POPDEN=16;       % people per km2
avghh=16;        % average agent size (people/agent)
hhreal=4;
time2mkt=1-MKTACC;
%%%%%%%%%%%%%%%%%%   AGENTS   %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
Nagents=(POPDEN*100)/avghh;
hhsize=avghh*ones(Nagents,1);
PARCEL=sqrt((NLENGTH*NWIDTH)/Nagents);
mktina=0.25;
alpha=min(mktina,0.75);

%%%% Risk Preferences %%%%
cemean=0.5;
cevar=0.1;

%%%% Expectation models %%%%
NUMMODEL=20;
PRODCLASS=6;
PRICECLASS=5;
MAXMEANMODEL=10;
MAXCYCLEMODEL=10;
MAXPROJECT=10;
% DELTA=1/5;
DELTA=0.6;

nfwageparm=metaparmset(mr,1,popden_count);
farmcostparm=metaparmset(mr,2,popden_count);
nfarmcostparm=metaparmset(mr,3,popden_count);
croppriceparm=metaparmset(mr,4,popden_count);

pconv=0.1836;

submin=hhsize/2+hhsize/4;
%Source: Evans et al. (2001); subsistence needs are between 2-4 ha
%Two adults per house count as full labor/subreq
%Two kids per house coutn as half "/"

homemin=15*submin(1);  %Includes leisure, non-active, home upkeep, and home industry (e.g. Ellis, 1993)
homeavg=52*(1-MKTINF);
Ltot=52*(hhsize/2+hhsize/4)-homemin;
deltapref=0.6;    %rate at which agents shift livelihood strategies

%%%%%%%%%%%%%%%%%% Biophysical factors %%%%%%%%%%%%%%%%%%%%
Nuse=6; %[intensive crop,extensive crop,pasture,forest,fallow(non-use),settlement]
Nuse_nonprod=3;  %[off-farm,leisure]
Neihood=10;

bu2kg=27.22;    %bushel of wheat = 27.22kg
% %%% kg/ha
PRODLAND=ones(NLENGTH,NWIDTH);
meanPROD=zeros(Nuse,1);
meanPROD(1)=3500;   %FAOSTAT
meanPROD(2)=meanPROD(1)*0.77;   %Ratio from Siebert and Doll (2010) irrigated/rainfed yeild
meanPROD(3)=9600/5.7;   %Yield from Tyson et al. (1990), conversion factor to GE Jahnke (1982)
meanPROD(4)=-1000;   %~64 bu/ha
meanPROD(5)=-1000;
meanPROD(6)=100000;

maxPROD=zeros(Nuse,1);
maxPROD(1)=meanPROD(1)*1.1;
maxPROD(2)=meanPROD(2)*1.1;
maxPROD(3)=meanPROD(3)*1.1;
maxPROD(4)=meanPROD(4)*1.1;
maxPROD(5)=meanPROD(5)*1.1;
maxPROD(6)=100000;

DEGRAD=zeros(Nuse,1);       %Degradation rate do to use, no inputs
DEGRAD(1)=0;    
DEGRAD(2)=0.17;     % Lal (1989), Tian et al.(2005)   
DEGRAD(3)=0.18;     %Tyson et al.(1990)

REGEN=zeros(Nuse,1);
REGEN(1)=0;     %Regeneration rate in fallow, no inputs
REGEN(2)=0.04;  %About 4%, from Tiessen et al. (1992)
REGEN(3)=0.50;  %Can be approximated with Tyson et al.(1990)

PRODVAR=zeros(Nuse,1);
PRICEVAR=zeros(Nuse,1);

SUBREQ=860*submin;  %Penning de Vries et al. (1997)- Moderate diet, includes feed to livestock
%Assuming approx.400 kg wheat(gross dry weight)/perspn*yr, 3,000 Kcal/day
% Wenhua, L. in Gilbert and Braat (1991, p142)
% Stephenne and Lambin (2001, p148)- 4600kg/yr for livestock

%%%%%%%%%%%%%%%%%% Land-Use Inputs %%%%%%%%%%%%%%%%%%%%%%%%
% Units in labor (person-weeks/ha/year)
% Increased production and labor with fertilizer inputs
ext2extcost=3;
tree2extcost=3;
laborcosts=[4   3       2.5     1000    0   1000;  
            4   ext2extcost       2.5     1000    0   1000;  
            4.5 2       1.5     1000    0   1000; 
            8   tree2extcost       4.5     1000    0   1000; 
            7   2       4.5     1000    0   1000; 
            1000 1000   1000    1000    1000 0];
inputcosts=zeros(size(laborcosts));
inputcosts(1:Nuse-1,1)=1;

%%%% Commodity Prices %%%%
% croppriceparm=2;
% croppriceparm=2.04;
pcrops=(3.5/bu2kg)*MKTINF^croppriceparm;
ppasture=(3.5/bu2kg)*MKTINF^croppriceparm;
pforest=0*MKTINF;
pfallow=0*MKTINF;
phouse=0*MKTINF;

pinput=0.75*pcrops;

nonfarmwage=6*5.75*5; % $5.75 minimum wage
minwage=nonfarmwage*MKTINF^nfwageparm;
farmcosts=farmcostparm*nonfarmwage/(1+MKTACC);
nonfarmcosts=nfarmcostparm*(1-MKTACC)*nonfarmwage;

inkindparm=0.25;
INCREQ=zeros(Nagents,TMAX);
INCLABOR=zeros(Nagents,TMAX);
INCREQ_opt=zeros(Nagents,TMAX);
INCLABOR_opt=zeros(Nagents,TMAX);
incgoal=zeros(Nagents,TMAX);
inclevel=zeros(Nagents,TMAX);
grossinc=zeros(Nagents,TMAX);
grossinc_opt=zeros(Nagents,TMAX);
incgoal_opt=zeros(Nagents,TMAX);
A=zeros(Nagents,TMAX);
A_opt=zeros(Nagents,TMAX);
exptcultarea=zeros(Nagents,TMAX);
exptcultarea_opt=zeros(Nagents,TMAX);
FARMWAGE=zeros(Nagents,Nuse,TMAX);
FARMWAGE_opt=zeros(Nagents,Nuse,TMAX);
NFARMWAGE=zeros(Nagents,TMAX);
NFARMWAGE_opt=zeros(Nagents,TMAX);
foodprod=zeros(Nagents,TMAX);
ifindexptsublab=zeros(Nagents,TMAX);
ifindexptmktlab=zeros(Nagents,TMAX);
ifindmaxsublab=zeros(Nagents,TMAX);
ifindmaxmktlab=zeros(Nagents,TMAX);
ifindsublab=zeros(Nagents,TMAX);
ifindmktlab=zeros(Nagents,TMAX);
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
%@@@@@@@@@@@@@@@@@@@@@@@@    VARIABLES    @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
%@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

LUpct=zeros(Nuse,TMAX);
LUpct_opt=zeros(Nuse,TMAX);

%%%%%%%%%%%%%%%%%% Biophysical factors %%%%%%%%%%%%%%%%%%%%

%%%% Initial Land Cover %%%%
LAND=zeros(NLENGTH,NWIDTH);

DEM=peaks(NLENGTH);
DEM(DEM < 0)=0;
DEM(DEM > 4)=4;
R=makerefmat('RasterSize',size(DEM));
[aspect,slope,gradN,gradE]=gradientm(DEM,R);
rainparm=0.75;
RAIN=ones(size(LAND))*rainparm;   %Consistent with GAEZ land suitability for area with one growing season
SOIL=1000*slope;
raingrad=RAIN;
slopegrad_ag=1-(max(min(-0.0012*SOIL.^3+0.0981*SOIL.^2+0.8192*SOIL-3.183,100),0))/100;
slopegrad_pas=1-(max(min(-0.0016*SOIL.^3+0.1636*SOIL.^2-1.931*SOIL+1.702,100),0))/100;

CROPsuit=zeros(NLENGTH,NWIDTH,3);
CROPsuit(:,:,1)=maxPROD(2)*raingrad.*slopegrad_ag; %Suitability for Ag
CROPsuit(:,:,2)=maxPROD(3)*raingrad.*slopegrad_pas;    %Suitability for Pasture
slopecon=(SOIL > 15);
CROPsuit(:,:,3)=maxPROD(2)*2*slopecon;         %Forest

[suitprod,suitcrop]=max(CROPsuit,[],3);

%%%% Land Suitability %%%%
Landsuit=zeros(size(LAND));
LU=zeros(NLENGTH,NWIDTH,TMAX);
LU_opt=zeros(NLENGTH,NWIDTH,TMAX);
% ENTER LAND USE LEGACY
timeinuse=ones(size(LAND));
timeoutprod=zeros(size(LAND));
timeinprod=zeros(size(LAND));

%%%% Land Suitability %%%%
% Initial fallow pct based on Turner et al. (1977) equation
falfac=(100-10^(1.06+0.0049*POPDEN))/100;

icropsint=(suitcrop == 1);
icropsext=(suitcrop == 1);
ipasture=(suitcrop == 2);
iforest=(suitcrop == 3);
icroplist=find(icropsint==1 | icropsext == 1); %list for randomly choosing fallow
ipastlist=find(ipasture==1);
Landsuit(icropsint)=5;  %1=intensive crop, 2=extensive crops, 3==pasture, 4=forest,fallow=5
Landsuit(ipasture)=3;
Landsuit(iforest)=4;

itrees=find(Landsuit==4);
icropsext=itrees(ceil(length(itrees)*rand(ceil((1-falfac)*length(itrees)),1)));
Landsuit(icropsext)=2;

timeinuse(itrees)=floor(1+(15-1)*rand(length(itrees),1));

subLandprod=zeros(size(Landsuit));

subLandsuit=Landsuit;
icropinitial=icroplist(ceil(rand(1,round((1-falfac)*length(icroplist)))*length(icroplist)));
subLandsuit(icropinitial)=2;
timeinuse(icropinitial)=1;

for t_0=1:TSTART+1
    LU(:,:,t_0)=subLandsuit;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%   Land Use Agents   %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%   AGENTS   %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

OWN=zeros(NLENGTH,NWIDTH);
U=zeros(NLENGTH,NWIDTH,TMAX);
EUsub=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
EUmkt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
EUsub_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
EUmkt_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);

STORE=zeros(Nagents,2,TMAX);    %[subsistence money_income]
STORE(:,1,1:TSTART)=repmat(SUBREQ,1,TSTART);
farmincreq=zeros(Nagents,TMAX);
foodincreq=zeros(Nagents,TMAX);
aspincreq=zeros(Nagents,TMAX);
foodbalance=zeros(Nagents,TMAX);

STORE_opt=zeros(Nagents,2,TMAX);    %[subsistence money_income]
STORE_opt(:,1,1:TSTART)=repmat(SUBREQ,1,TSTART);
farmincreq_opt=zeros(Nagents,TMAX);
foodincreq_opt=zeros(Nagents,TMAX);
aspincreq_opt=zeros(Nagents,TMAX);
foodbalance_opt=zeros(Nagents,TMAX);

altU=zeros(PARCEL,PARCEL,Nuse,TMAX);
Labor=zeros(Nagents,3,TMAX);
Labor_opt=zeros(Nagents,3,TMAX);
Farmlabor=zeros(Nagents,2,TMAX);
Farmlabor_opt=zeros(Nagents,2,TMAX);
laborprefs=zeros(Nagents,TMAX); %[alpha_leis alpha_farm alpha_nonfarm]
inkindwage=zeros(Nagents,TMAX);
inkindwage_opt=zeros(Nagents,TMAX);

laborprefs_opt=zeros(Nagents,TMAX); %[alpha_leis alpha_farm alpha_nonfarm]
sigmahome=zeros(Nagents,TMAX);
sigmahome_opt=zeros(Nagents,TMAX);

SUBLABOR=zeros(Nagents,TMAX);
SUBLABOR_opt=zeros(Nagents,TMAX);
PCTINC=zeros(Nagents,Nuse,TMAX);
PCTINC_opt=zeros(Nagents,TMAX);
MARGPROD=zeros(Nagents,2,TMAX); %[meanmargyield meanmargprod]
foodprice=zeros(Nagents,TMAX);
shadowprice=zeros(Nagents,TMAX);
foodprice_opt=zeros(Nagents,TMAX);
shadowprice_opt=zeros(Nagents,TMAX);
efood=zeros(Nagents,TMAX);
efood_opt=zeros(Nagents,TMAX);

CEneut=zeros(Nagents,TMAX);   %certainty equivalent of farmwage
CEra=zeros(Nagents,TMAX);   %certainty equivalent of off-farmwage

CEneut_opt=zeros(Nagents,TMAX);
CEra_opt=zeros(Nagents,TMAX);

pctfarminc=zeros(Nagents,TMAX);
pctofffarminc=zeros(Nagents,TMAX);

pctfarminc_opt=zeros(Nagents,TMAX);
pctofffarminc_opt=zeros(Nagents,TMAX);

savedState=stream.State;
stream.Substream=86;
stream.State=repeatstate1;

CE=max(min(cemean+cevar*randn(Nagents,1),1),0);

AVGyield=zeros(NLENGTH,NWIDTH,Nuse);
AVGprod=zeros(NLENGTH,NWIDTH,Nuse);
AVGexptyield=zeros(NLENGTH,NWIDTH,Nuse);
AVGexptprod=zeros(NLENGTH,NWIDTH,Nuse);
MAXyield=zeros(NLENGTH,NWIDTH,TMAX);
MAXprod=zeros(NLENGTH,NWIDTH,TMAX);
MAXyieldlu=zeros(NLENGTH,NWIDTH,TMAX);
MAXprodlu=zeros(NLENGTH,NWIDTH,TMAX);

prodproj = zeros(NLENGTH,NWIDTH,NUMMODEL,Nuse);
proderror = zeros(NLENGTH,NWIDTH,NUMMODEL,Nuse);
prodbestSAVE=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
iprodbestSAVE=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
prodprojSAVE=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
prodprojbestSAVE=zeros(NLENGTH,NWIDTH,NUMMODEL,Nuse);
prodmodelbestSAVE=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
prodmodelSAVE=zeros(NLENGTH,NWIDTH,Nuse,TMAX);

priceproj = zeros(Nagents,NUMMODEL,Nuse);
priceerror = zeros(Nagents,NUMMODEL,Nuse);
pricebestSAVE=zeros(Nagents,Nuse,TMAX);
ipricebestSAVE=zeros(Nagents,Nuse,TMAX);
priceprojSAVE=zeros(Nagents,Nuse,TMAX);
priceprojbestSAVE=zeros(Nagents,NUMMODEL,Nuse);
pricemodelbestSAVE=zeros(Nagents,Nuse,TMAX);
pricemodelSAVE=zeros(Nagents,Nuse,TMAX);
    
%%%%%% Expectation Models %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% YIELD EXPECTATIONS
prodmodel = ceil(PRODCLASS*rand(NLENGTH,NWIDTH,NUMMODEL));
for i = 1:PRODCLASS
    strl = sprintf('prodclass%d = find(prodmodel == %d);',i,i);
    eval(strl);
end

aa = zeros(NLENGTH,NWIDTH,NUMMODEL);    %production models
for i = 1:PRODCLASS
    if i == 1
        % mirror model
        aa(prodclass1) = rand(1); % fraction that pred is away from 1/2 from mirror image
    elseif i == 2
        % mean model
        aa(prodclass2) = ceil(MAXMEANMODEL*rand(length(prodclass2),1));
    elseif i == 3
        %cycle model
        aa(prodclass3) = ceil(MAXCYCLEMODEL*rand(length(prodclass3),1));
    elseif i == 4
        % projection model
        aa(prodclass4) = ceil(2+((MAXPROJECT-1)-2)*rand(length(prodclass4),1));
    elseif i == 5
        % rescale model
        aa(prodclass5) = 2*rand(length(prodclass5),1);
    elseif i == 6
        %regional trends
        aa(prodclass6) = ceil(MAXMEANMODEL*rand(length(prodclass6),1));
    end
end


%%% PRICE EXPECTATIONS
pricemodel = ceil(PRICECLASS*rand(Nagents,NUMMODEL));
for i = 1:PRICECLASS
    strl = sprintf('priceclass%d = find(pricemodel == %d);',i,i);
    eval(strl);
end

bb = zeros(Nagents,NUMMODEL);    %price models
for i = 1:PRICECLASS
    if i == 1
        % mirror model
        bb(priceclass1) = rand(1); % fraction that pred is away from 1/2 from mirror image
    elseif i == 2
        % mean model
        bb(priceclass2) = ceil(MAXMEANMODEL*rand(length(priceclass2),1));
    elseif i == 3
        %cycle model
        bb(priceclass3) = ceil(MAXCYCLEMODEL*rand(length(priceclass3),1));
    elseif i == 4
        % projection model
        bb(priceclass4) = ceil(2+((MAXPROJECT-1)-2)*rand(length(priceclass4),1));
    elseif i == 5
        % rescale model
        bb(priceclass5) = 2*rand(length(priceclass5),1);
    end
end

for c=1:PARCEL
    iparcelcol=PARCEL*(c-1)+(1:PARCEL);
    for r=1:PARCEL
        iparcelrow=PARCEL*(r-1)+(1:PARCEL);
        OWN(iparcelrow,iparcelcol)=(c-1)*PARCEL+r;
    end
end

stream.Substream=mrun;
stream.State=savedState;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%   Base Productivity   %%%%%%%%%%%%
BASEPROD=zeros(NLENGTH,NWIDTH,Nuse,TMAX);   %Productivity of land under use,constant climate
PROD=zeros(NLENGTH,NWIDTH,Nuse,TMAX);       %Realized productivity of land, subject to climate variability
EXPTPROD=zeros(NLENGTH,NWIDTH,Nuse,TMAX);   % Expected productivity
Landprod=zeros(NLENGTH,NWIDTH,TMAX);
margsubprod=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
margmktprod=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
submargmean=zeros(Nuse,TMAX);
mktmargmean=zeros(Nuse,TMAX);
margsubprod_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
margmktprod_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
ltavgyield=zeros(Nagents,TMAX);
ltavgprod=zeros(Nagents,TMAX);
ltmargprod=zeros(Nagents,TMAX);
ltavgscape=zeros(NLENGTH,NWIDTH,3);         %Only productive uses
iutranscost=zeros(1,Nuse);
iulaborcost=zeros(1,Nuse);
iuinputcost=zeros(1,Nuse);
iupaidlabor=zeros(Nuse,TMAX);

BASEPROD(:,:,1,1)=meanPROD(1)*slopegrad_ag.*raingrad; %slope as a proxy for ag prod, citation Hoffman et. al. (2004)
BASEPROD(:,:,2,1)=meanPROD(2)*slopegrad_ag.*raingrad;
BASEPROD(:,:,3,1)=meanPROD(3)*slopegrad_pas.*raingrad;

subtree=BASEPROD(:,:,4,1);
subtree(itrees)=meanPROD(4)*1./(1+exp(-0.25*(timeinuse(itrees))));  %Productivity of forest based on age
BASEPROD(:,:,4,2:TSTART+1)=repmat(subtree,[1 1 TSTART]);
subprod1=BASEPROD(:,:,1,1);
subprod2=BASEPROD(:,:,2,1);
subprod3=BASEPROD(:,:,3,1);
subprod4=BASEPROD(:,:,4,1);

subprod1(icropsint)=subprod1(icropsint)-DEGRAD(1)*subprod1(icropsint).*(timeinuse(icropsint)-1);
BASEPROD(:,:,1,2:TSTART+1)=repmat(subprod1,[1 1 TSTART]);
subprod2(icropsext)=subprod2(icropsext)-DEGRAD(2)*subprod2(icropsext).*(timeinuse(icropsext)-1);
BASEPROD(:,:,2,2:TSTART+1)=repmat(subprod2,[1 1 TSTART]);
subprod3(ipasture)=subprod3(ipasture)-DEGRAD(3)*subprod3(ipasture).*(timeinuse(ipasture)-1);
BASEPROD(:,:,3,2:TSTART+1)=repmat(subprod3,[1 1 TSTART]);
DWELL=zeros(Nagents,1);
%%%%% Assignment of Dwelling Location %%%%%
for ia=1:Nagents
   iparcel=find(OWN == ia);
   [parrow,parcol]=ind2sub([NLENGTH NWIDTH],iparcel);
   iparrow=unique(parrow);
   iparcol=unique(parcol);
   parprod=mean(BASEPROD(iparrow,iparcol,1:2,1),3);
   
   [imaxrow,imaxcol]=find(parprod == max(max(parprod)),1,'first');
   BASEPROD(iparrow(imaxrow),iparcol(imaxcol),1:Nuse-Nuse_nonprod,:)=-10000*ones(Nuse-Nuse_nonprod,TMAX);
   DWELL(ia)=sub2ind(size(LAND),iparrow(imaxrow),iparcol(imaxcol));
end
subprod6=BASEPROD(:,:,6,1);
subprod6(DWELL)=100000;
BASEPROD(:,:,6,1:TSTART+1)=repmat(subprod6,[1 1 TSTART+1]);

subLU=LU(:,:,TSTART);
subLU(DWELL)=6;
LU(:,:,1:TSTART)=repmat(subLU,[1 1 TSTART]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   Distance matrices   %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DISTANCE=zeros(NLENGTH,NWIDTH,Nagents);
travelcost=zeros(size(DISTANCE));
tcparm=30;
for ia=1:Nagents
    [icenterrow,icentercol]=ind2sub(size(LAND),DWELL(ia));

    for col=1:NWIDTH
        dist2hznnei(1:NLENGTH,col)=abs(col-icentercol).*ones(NLENGTH,1);
    end
    
    for row=1:NWIDTH
        dist2vrtnei(row,1:NWIDTH)=abs(row-icenterrow).*ones(1,NLENGTH);
    end
    
    for col=1:NWIDTH
        for row=1:NLENGTH
            DISTANCE(row,col,ia)=sqrt(dist2hznnei(row,col)^2+dist2vrtnei(row,col)^2);
        end
    end
    
    travelcost(:,:,ia)=DISTANCE(:,:,ia)./(tcparm*PARCEL);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


prodvar_all=PRODVAR(1:Nuse)*randn(1,TMAX);

for k=1:TSTART+1
    for iu=1:Nuse
        PROD(:,:,iu,k)=min(repmat(prodvar_all(iu,k),NLENGTH,NWIDTH).*...
            BASEPROD(:,:,iu,k)+BASEPROD(:,:,iu,k),maxPROD(iu));
    end
end

subLandprod(Landsuit==1)=subprod1(Landsuit==1);
subLandprod(Landsuit==2)=subprod2(Landsuit==2);
subLandprod(Landsuit==3)=subprod3(Landsuit==3);
subLandprod(Landsuit==4)=subprod3(Landsuit==4);

subLU=LU(:,:,TSTART);

ALTSET=zeros(NLENGTH,NWIDTH,Nuse+1);
for i=1:Nuse
    ALTSET(:,:,i)=i*ones(NLENGTH,NWIDTH);
end
ALTSET(:,:,Nuse+1)=LU(:,:,TSTART);

%%%%%%%%%%%%%%%%%%%%%%%
luprefs=zeros(Nagents,Nuse,TMAX);
luprefs_opt=zeros(Nagents,Nuse,TMAX);

for ia=1:Nagents
   iown=find(OWN==ia);
   subLU=LU(:,:,TSTART);
   sublu(1:length(iown))=subLU(iown);
   prodplots=length(find(sublu == 1 | sublu == 2 | sublu == 3));
   PCTINC(ia,1,1:TSTART)=length(find(sublu == 1))/prodplots;
   PCTINC(ia,2,1:TSTART)=length(find(sublu == 2))/prodplots;
   PCTINC(ia,3,1:TSTART)=length(find(sublu == 3))/prodplots;
   
   PCTINC_opt(ia,1,1:TSTART)=length(find(sublu == 1))/prodplots;
   PCTINC_opt(ia,2,1:TSTART)=length(find(sublu == 2))/prodplots;
   PCTINC_opt(ia,3,1:TSTART)=length(find(sublu == 3))/prodplots;
   
   for iu=1:Nuse-Nuse_nonprod
       luprefs(ia,iu,1)=1/(Nuse-Nuse_nonprod);
   end
   luprefs(ia,6,1)=1;
end
luprefs(:,:,1:TSTART)=repmat(luprefs(:,:,1),[1 1 TSTART]);
luprefs(:,:,1:TSTART)=luprefs(:,:,1:TSTART);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%    Land Use Inputs    %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

transmat=reshape((1:Nuse^2),Nuse,Nuse);

%%%%%%%%%%%%%%%%%   Commodity Prices   %%%%%%%%%%%%%%
WTAPRICE=zeros(Nagents,Nuse,TMAX);
Price=zeros(Nuse,TMAX);
Price(1,1)=pcrops;
Price(2,1)=pcrops;
Price(3,1)=ppasture;
Price(4,1)=pforest;
Price(5,1)=pfallow;
Price(6,1)=phouse;
for it=2:TMAX
    Price(:,it)=max(Price(:,it-1)+Price(:,it-1).*PRICEVAR(:,1).*randn(Nuse,1),0);
end
Price(2,:)=Price(1,:);
pfert=mean(Price(1,:));

%%%% Certainty Equivalent for risk-neutral activities %%%%
% CEneut is the risk-neutral certainty equivalent of the land user's most
% profitable land use (represented as the mean yield and price)
transcost=laborcosts;
labcost=laborcosts;
transcost(transcost == 1000)=0;
labcost(labcost == 1000)=0;
transcost=transcost+inputcosts;
iuinputcost=inputcosts(1,:);
for iu=1:Nuse-Nuse_nonprod
    iutranscost(iu)=mean(transcost(transcost(1:3,iu) ~= 0,iu));
    iulaborcost(iu)=mean(labcost(labcost(1:3,iu) ~= 0,iu));
end
iutranscost(Nuse-Nuse_nonprod+1:Nuse)=1000;
iulaborcost(Nuse-Nuse_nonprod+1:Nuse)=1000;

foodprod(:,1:TSTART)=repmat(SUBREQ,1,TSTART);

for ia=1:Nagents
    iown=find(OWN==ia);
    iprodland=iown(~ismember(iown,DWELL));
    sublus=LU(:,:,TSTART);
    subtc=travelcost(:,:,ia);
    [subrow,subcol]=ind2sub([NLENGTH NWIDTH],iprodland);
    for ic=1:length(iprodland)
        avgyield(1:Nuse-Nuse_nonprod,1)=mean(PROD(subrow(ic),subcol(ic),...
            1:Nuse-Nuse_nonprod,1:TSTART+1),4);
        avgprod=avgyield.*mean(Price(1:Nuse-Nuse_nonprod,1:TSTART+1),2);
        AVGyield(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgyield;
        AVGprod(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgprod;
    end
    
    for iu=1:Nuse-Nuse_nonprod
        subay=AVGyield(:,:,iu);
        subap=AVGprod(:,:,iu);
        meanyield(iu)=mean(subay(iprodland));
        margmeanyield(iu)=meanyield(iu)./(iulaborcost(iu)+mean(subtc(iprodland)));
        meanprod(iu)=mean(subap(iprodland));
        FARMWAGE(ia,iu,1:TSTART)=max(meanprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt),0);
        margmeanprod(iu)=(meanprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt))/(iulaborcost(iu)+mean(subtc(iprodland)));
        
        submaxp=PROD(:,:,iu,TSTART);
        meanmaxyield(iu)=mean(submaxp(iprodland));
        margmeanmaxyield(iu)=meanmaxyield(iu)./(iulaborcost(iu)+mean(subtc(iprodland)));
        meanmaxprod(iu)=mean(submaxp(iprodland))*Price(iu,TSTART);
        FARMWAGE_opt(ia,iu,1:TSTART+1)=max(meanmaxprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt),0);
        margmeanmaxprod(iu)=(meanmaxprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt))/(iulaborcost(iu)+mean(subtc(iprodland)));   
    end
    
    ifindsublab(ia,TSTART)=find(margmeanyield == max(margmeanyield));
    ifindmktlab(ia,TSTART)=find(FARMWAGE(ia,1:Nuse-Nuse_nonprod,TSTART) == ...
        max(FARMWAGE(ia,1:Nuse-Nuse_nonprod,TSTART)),1,'first');
    ifindmaxsublab(ia,TSTART)=find(margmeanmaxyield == max(margmeanmaxyield));
    ifindmaxmktlab(ia,TSTART)=find(FARMWAGE_opt(ia,1:Nuse-Nuse_nonprod,TSTART) == ...
        max(FARMWAGE_opt(ia,1:Nuse-Nuse_nonprod,TSTART)),1,'first');

    foodprice(ia,1:TSTART)=(1-CE(ia))*((2-MKTACC)*farmcosts*...
            (iuinputcost(ifindsublab(ia,TSTART))+time2mkt));
    shadowprice(ia,1:TSTART)=0.5*farmcosts*(iuinputcost(ifindsublab(ia,TSTART))+time2mkt);
    NFARMWAGE(ia,1:TSTART)=max((minwage-nonfarmcosts)*...
        (iulaborcost(ifindmktlab(ia,TSTART))+mean(subtc(iprodland))),0);
    %%%%%%%%%%%%%%%   Initial Labor Allocation   %%%%%%%%%%%%%%%%%%%%%%%%%%
    Labor(ia,2,1:TSTART)=ceil(SUBREQ(ia)./margmeanyield(ifindsublab(ia,TSTART))+...
        mean(subtc(iprodland))*submin(ia));
    if FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART) == 0 && NFARMWAGE(ia,TSTART) == 0
        pctfarminc(ia,1:TSTART)=0;
        pctofffarminc(ia,1:TSTART)=0;
        Labor(ia,1,1:TSTART)=Labor(ia,1,1:TSTART)+Labor(ia,3,1:TSTART);
        Labor(ia,3,1:TSTART)=0;
    else
        pctfarminc(ia,1:TSTART)=repmat(max(min((0.5*FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART))/...
            (0.5*FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART)+CE(ia)*NFARMWAGE(ia,TSTART)),1),0),1,TSTART);
        pctofffarminc(ia,1:TSTART)=1-pctfarminc(ia,1:TSTART);
    end
    Labor(ia,3,1:TSTART)=ceil(pctofffarminc(ia,TSTART)*Labor(ia,2,1:TSTART));
    Labor(ia,1,1:TSTART)=repmat(Ltot(ia)-sum(Labor(ia,2:3,TSTART)),1,TSTART);  
    laborprefs(ia,1:TSTART)=Labor(ia,1,1:TSTART)/Ltot(ia);

    exptcultarea(ia,1:TSTART)=repmat(ceil(SUBREQ(ia)/meanyield(ifindsublab(ia,TSTART))),1,TSTART);
    
    SUBLABOR(ia,1:TSTART)=repmat(ceil(SUBREQ(ia)./margmeanyield(ifindsublab(ia,TSTART))+...
        mean(subtc(iprodland))*exptcultarea(ia,TSTART)),1,TSTART);
    
    inkindwage(ia,1:TSTART)=farmcosts*iuinputcost(ifindsublab(ia,TSTART))*...
        (SUBLABOR(ia,1:TSTART)/iulaborcost(ifindsublab(ia,TSTART)));
    
    A(ia,1:TSTART)=repmat((1+MKTINF)*FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART)*...
        ceil(SUBREQ(ia)/margmeanyield(ifindsublab(ia,TSTART))),1,TSTART);
    farmincreq(ia,1:TSTART)=repmat(farmcosts*((ceil(pctfarminc(ia,TSTART)*...
        Labor(ia,2,TSTART))/iulaborcost(ifindmktlab(ia,TSTART)))*iuinputcost(ifindmktlab(ia,TSTART))+...
        (SUBLABOR(ia,TSTART)/iulaborcost(ifindsublab(ia,TSTART)))*iuinputcost(ifindsublab(ia,TSTART))),1,TSTART);
    if STORE(ia,1,TSTART) > 0
            foodincreq(ia,1:TSTART)=0;
    else
        foodincreq(ia,1:TSTART)=Price(ifindsublab(ia,TSTART),TSTART).*...
            abs(foodbalance(ia,1:TSTART));
    end
    INCREQ(ia,1:TSTART)=farmincreq(ia,1:TSTART)+foodincreq(ia,1:TSTART)+...
        A(ia,1:TSTART);
    STORE(ia,2,1:TSTART)=repmat(INCREQ(ia,TSTART),1,TSTART);
    STORE(ia,2,TSTART+1)=STORE(ia,2,TSTART)+A(ia,TSTART);
    
    if NFARMWAGE(ia,TSTART) == 0 && FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART) == 0
        INCLABOR(ia,1:TSTART)=0;
    elseif NFARMWAGE(ia,TSTART) == 0
        INCLABOR(ia,1:TSTART)=repmat(ceil((pctfarminc(ia,TSTART)*...
            INCREQ(ia,TSTART))/FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART)),1,TSTART);
    elseif FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART) == 0
        INCLABOR(ia,1:TSTART)=repmat(ceil((pctofffarminc(ia,TSTART)*...
            INCREQ(ia,TSTART))/NFARMWAGE(ia,TSTART)),1,TSTART);
    else
        INCLABOR(ia,1:TSTART)=repmat(ceil((pctofffarminc(ia,TSTART)*...
            INCREQ(ia,TSTART))/NFARMWAGE(ia,TSTART)+(pctfarminc(ia,TSTART)*...
            INCREQ(ia,TSTART))/FARMWAGE(ia,ifindmktlab(ia,TSTART),TSTART)),1,TSTART);
    end
    
    %Allocation of sub vs market production
    Farmlabor(ia,1,1:TSTART)=SUBLABOR(ia,1:TSTART);
    Farmlabor(ia,2,1:TSTART)=min(ceil(pctfarminc(ia,TSTART)*INCLABOR(ia,TSTART)),...
        Labor(ia,2,TSTART)-SUBLABOR(ia,TSTART));
    
    %%%%%%%%%%%%%%%%%%   Optimal Routine   %%%%%%%%%%%%%%%%%%%%%%%%%%
    foodprice_opt(ia,1:TSTART)=((2-MKTACC)*farmcosts*(iuinputcost(ifindmaxsublab(ia,TSTART))+time2mkt));
    shadowprice_opt(ia,1:TSTART)=farmcosts*(iuinputcost(ifindmaxsublab(ia,TSTART))+time2mkt);
    NFARMWAGE_opt(ia,1:TSTART)=max((minwage-nonfarmcosts)*...
        (iulaborcost(ifindmaxmktlab(ia,TSTART))+mean(subtc(iprodland))),0);
    Labor_opt(ia,2,1:TSTART)=ceil(SUBREQ(ia)./margmeanmaxyield(ifindmaxsublab(ia,TSTART))+...
        mean(subtc(iprodland))*submin(ia));
    if FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART) == 0 && NFARMWAGE_opt(ia,TSTART) == 0
        pctfarminc_opt(ia,1:TSTART)=0;
        pctofffarminc_opt(ia,1:TSTART)=0;
        Labor_opt(ia,1,1:TSTART)=Labor_opt(ia,1,1:TSTART)+Labor_opt(ia,3,1:TSTART);
        Labor_opt(ia,3,1:TSTART)=0;
    else
        pctfarminc_opt(ia,1:TSTART)=repmat((FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART))/...
            (FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART)+NFARMWAGE_opt(ia,TSTART)),1,TSTART);
        pctofffarminc_opt(ia,1:TSTART)=1-pctfarminc_opt(ia,1:TSTART);
    end
    
    Labor_opt(ia,3,1:TSTART)=ceil(pctofffarminc_opt(ia,TSTART)*Labor_opt(ia,2,1:TSTART));
    Labor_opt(ia,1,1:TSTART)=repmat(Ltot(ia)-sum(Labor_opt(ia,2:3,TSTART)),1,TSTART);
    laborprefs_opt(ia,1:TSTART)=Labor_opt(ia,1,1:TSTART)/Ltot(ia);
    
    SUBLABOR_opt(ia,1:TSTART)=repmat(ceil(SUBREQ(ia)./(margmeanmaxyield(ifindmaxsublab(ia,TSTART)))+...
            mean(subtc(iprodland))*exptcultarea_opt(ia,TSTART)),1,TSTART);
    inkindwage_opt(ia,1:TSTART)=farmcosts*iuinputcost(ifindmaxsublab(ia,TSTART))*...
        (SUBLABOR_opt(ia,1:TSTART)/iulaborcost(ifindmaxsublab(ia,TSTART)));     
    A_opt(ia,1:TSTART)=repmat((1+MKTINF)*FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART)*...
        ceil(SUBREQ(ia)/margmeanmaxyield(ifindmaxsublab(ia,TSTART))),1,TSTART);
    farmincreq(ia,1:TSTART)=repmat(farmcosts*((ceil(pctfarminc_opt(ia,TSTART)*...
        Labor_opt(ia,2,TSTART))/iulaborcost(ifindmaxmktlab(ia,TSTART)))*iuinputcost(ifindmaxmktlab(ia,TSTART))+...
        (SUBLABOR_opt(ia,TSTART)/iulaborcost(ifindmaxsublab(ia,TSTART)))*iuinputcost(ifindmaxsublab(ia,TSTART))),1,TSTART);

    if STORE(ia,1,TSTART) > 0
            foodincreq_opt(ia,1:TSTART)=0;
    else
        foodincreq(ia,1:TSTART)=Price(ifindmaxsublab(ia,TSTART),TSTART).*...
            abs(foodbalance(ia,1:TSTART));
    end
    INCREQ_opt(ia,1:TSTART)=farmincreq_opt(ia,1:TSTART)+foodincreq_opt(ia,1:TSTART)+...
            A_opt(ia,1:TSTART);
    STORE_opt(ia,2,1:TSTART+1)=repmat(INCREQ_opt(ia,TSTART),1,TSTART+1);
    
    if NFARMWAGE_opt(ia,TSTART) == 0 && FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART) == 0
        INCLABOR_opt(ia,1:TSTART)=0;
    elseif NFARMWAGE_opt(ia,TSTART) == 0
        INCLABOR_opt(ia,1:TSTART)=repmat(ceil((pctfarminc_opt(ia,TSTART)*...
            INCREQ_opt(ia,TSTART))/FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART)),1,TSTART);
    elseif FARMWAGE(ia,ifindmaxmktlab(ia,TSTART),TSTART) == 0
        INCLABOR_opt(ia,1:TSTART)=repmat(ceil((pctofffarminc_opt(ia,TSTART)*...
            INCREQ_opt(ia,TSTART))/NFARMWAGE_opt(ia,TSTART)),1,TSTART);
    else
        INCLABOR_opt(ia,1:TSTART)=repmat(ceil((pctofffarminc_opt(ia,TSTART)*...
            INCREQ_opt(ia,TSTART))/NFARMWAGE_opt(ia,TSTART)+(pctfarminc_opt(ia,TSTART)*...
            INCREQ_opt(ia,TSTART))/FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),TSTART)),1,TSTART);
    end
        
    %Allocation of sub vs market production
    Farmlabor_opt(ia,1,1:TSTART)=SUBLABOR_opt(ia,1:TSTART);
    Farmlabor_opt(ia,2,1:TSTART)=min(ceil(pctfarminc_opt(ia,TSTART)*INCLABOR_opt(ia,TSTART)),...
        Labor_opt(ia,2,TSTART)-SUBLABOR_opt(ia,TSTART));
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    MARGPROD(ia,1,1:TSTART)=repmat(margmeanyield(ifindsublab(ia,TSTART)),1,TSTART);
    MARGPROD(ia,2,1:TSTART)=repmat(margmeanprod(ifindmktlab(ia,TSTART)),1,TSTART);
    
    CEneut(ia,1:TSTART)=0.5*FARMWAGE(ia,ifindmktlab(ia,TSTART),1:TSTART);
    CEneut_opt(ia,1:TSTART)=FARMWAGE_opt(ia,ifindmaxmktlab(ia,TSTART),1:TSTART);
        
    CEra(ia,1:TSTART)=CE(ia)*NFARMWAGE(ia,1:TSTART);
    CEra_opt(ia,1:TSTART)=NFARMWAGE_opt(ia,1:TSTART);
    
end

subscape=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
mktscape=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
altusecost=zeros(NLENGTH,NWIDTH,Nuse);
altuseprod=zeros(NLENGTH,NWIDTH,Nuse);
altmargprod=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
currentprod=zeros(NLENGTH,NWIDTH);

subscape_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
mktscape_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
altusecost_opt=zeros(NLENGTH,NWIDTH,Nuse);
altuseprod_opt=zeros(NLENGTH,NWIDTH,Nuse);
altmargprod_opt=zeros(NLENGTH,NWIDTH,Nuse,TMAX);
currentprod_opt=zeros(NLENGTH,NWIDTH);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Agents form initial expectations %%%%

iNagents=unique(OWN);
iNagents=iNagents(iNagents~=0);

%%% YIELD EXPECTATIONS
ti=TSTART+1;
for iuse=1:Nuse-Nuse_nonprod
    subprodproj=prodproj(:,:,:,iuse);
    subproderror=proderror(:,:,:,iuse);
    for icol=1:NWIDTH
        for irow=1:NLENGTH            
            iprodclass1=find(prodmodel(irow,icol,:)==1);
            iprodclass2=find(prodmodel(irow,icol,:)==2);
            iprodclass3=find(prodmodel(irow,icol,:)==3);
            iprodclass4=find(prodmodel(irow,icol,:)==4);
            iprodclass5=find(prodmodel(irow,icol,:)==5);
            iprodclass6=find(prodmodel(irow,icol,:)==6);            
            for i = 1:PRODCLASS
                if i == 1
                    % mimic models
                    subprodproj(irow,icol,iprodclass1) = PROD(irow,icol,iuse,ti)+(1-aa...
                        (irow,icol,iprodclass1)).*(0.5*PROD(irow,icol,iuse)-...
                        (PROD(irow,icol,iuse,ti)-PROD(irow,icol,iuse,ti-1)));
                elseif i == 2
                    % mean model
                    for jk = 1:length(iprodclass2)
                        subprodproj(irow,icol,iprodclass2(jk)) = mean(PROD(irow,icol,iuse,...
                            ti:-1:(ti-aa(irow,icol,iprodclass2(jk)))));
                    end
                elseif i == 3
                    %cycle model
                    subprodproj(irow,icol,iprodclass3) = PROD(irow,icol,iuse,...
                        ti-round(max(1,aa(irow,icol,iprodclass3))));
                elseif i == 4
                    % projection model
                    for jl = 1:length(iprodclass4)
                        indata=zeros(MAXMEANMODEL,1);
                        %Nonlinear Forecast
                        timespan=length(ti-(1+aa(irow,icol,iprodclass4(jl))):ti);
                        indata(1:timespan)=PROD(irow,icol,iuse,ti-(1+aa(irow,icol,iprodclass4(jl))):ti);
                        pcoef=polyfit(1:timespan,indata(1:timespan)',1);
                        pline=pcoef(1).*(1:timespan+1)+pcoef(2);
                        subprodproj(irow,icol,iprodclass4(jl))=pline(length(pline));
                    end
                elseif i == 5
                    % rescale model
                    subprodproj(irow,icol,iprodclass5) = aa(irow,icol,...
                        iprodclass5)*PROD(irow,icol,iuse,ti);
                elseif i == 6
                    irownei=max(min(irow-Neihood:irow+Neihood,NLENGTH),1);
                    icolnei=max(min(icol-Neihood:icol+Neihood,NLENGTH),1);
                    for jm=1:length(iprodclass6)
                        subprodproj(irow,icol,iprodclass6(jm)) = mean(mean(mean(PROD(irownei,icolnei,...
                            iuse,ti:-1:(ti-aa(irow,icol,iprodclass6(jm)))),4)));
                    end
                end
            end
            subproderror(irow,icol,:) = (1-DELTA)*subproderror(irow,icol,:)+...
                DELTA*abs(PROD(irow,icol,iuse,ti)-subprodproj(irow,icol,:));
            [prodbest iprodbest] = min(subproderror(irow,icol,:),[],3);
            prodbestSAVE(irow,icol,iuse,ti) = prodbest;
            iprodbestSAVE(irow,icol,iuse,ti) = iprodbest;
            prodprojSAVE(irow,icol,iuse,ti) = subprodproj(irow,icol,iprodbest);
            prodmodelSAVE(irow,icol,iuse,ti) = prodmodel(irow,icol,iprodbest);
            
            EXPTPROD(irow,icol,iuse,ti)=subprodproj(irow,icol,iprodbest);
        end
    end
    prodproj(:,:,:,iuse)=subprodproj;
    proderror(:,:,:,iuse)=subproderror;

%%%%% PRICE EXPECTATIONS %%%%%
    subpriceproj=priceproj(:,:,iuse);
    subpriceerror=priceerror(:,:,iuse);
    for na=1:length(iNagents)
        ipriceclass1=find(pricemodel(iNagents(na),:)==1);
        ipriceclass2=find(pricemodel(iNagents(na),:)==2);
        ipriceclass3=find(pricemodel(iNagents(na),:)==3);
        ipriceclass4=find(pricemodel(iNagents(na),:)==4);
        ipriceclass5=find(pricemodel(iNagents(na),:)==5);
        ipriceclass6=find(pricemodel(iNagents(na),:)==6);
        
        for i = 1:PRICECLASS
            if i == 1
                % mimic models
                subpriceproj(iNagents(na),ipriceclass1) = Price(iuse,ti)+(1-bb...
                    (iNagents(na),ipriceclass1)).*(0.5*Price(iuse,ti)-...
                    (Price(iuse,ti)-Price(iuse,ti-1)));
            elseif i == 2
                % mean model
                for jl = 1:length(ipriceclass2)
                    subpriceproj(iNagents(na),ipriceclass2(jl)) = mean(Price(iuse,...
                        ti:-1:(ti-bb(iNagents(na),ipriceclass2(jl)))));
                end
            elseif i == 3
                %cycle model
                subpriceproj(iNagents(na),ipriceclass3) = Price(iuse,...
                    ti-round(max(1,bb(iNagents(na),ipriceclass3))));
            elseif i == 4
                % projection model
                for jl = 1:length(ipriceclass4)
                    %Nonlinear Forecast
                    indata=Price(iuse,ti-(1+bb(iNagents(na),ipriceclass4(jl))):ti);
                    pcoef=polyfit(1:length(indata),indata,1);
                    pline=pcoef(1).*(1:length(indata)+1)+pcoef(2);
                    subpriceproj(iNagents(na),ipriceclass4(jl))=pline(length(pline));
                end
            elseif i == 5
                % rescale model
                subpriceproj(iNagents(na),ipriceclass5) = bb(iNagents(na),...
                    ipriceclass5)*Price(iuse,ti);
            end
        end
        subpriceerror(iNagents(na),:) = (1-DELTA)*subpriceerror(iNagents(na),:)+...
            DELTA*abs(Price(iuse,ti)-subpriceproj(iNagents(na),:));
        [pricebest ipricebest] = min(subpriceerror(iNagents(na),:),[],2);
        pricebestSAVE(iNagents(na),iuse,ti) = pricebest;
        ipricebestSAVE(iNagents(na),iuse,ti) = ipricebest;
        priceprojSAVE(iNagents(na),iuse,ti) = subpriceproj(iNagents(na),ipricebest);
        pricemodelSAVE(iNagents(na),iuse,ti) = pricemodel(iNagents(na),ipricebest);

        WTAPRICE(iNagents(na),iuse,ti)=subpriceproj(iNagents(na),ipricebest);
    end
    priceproj(:,:,iuse)=subpriceproj;
    priceerror(:,:,iuse)=subpriceerror;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

subexp=EXPTPROD(:,:,6,1);
subexp(DWELL)=1000;
EXPTPROD(:,:,6,:)=repmat(subexp,[1 1 TMAX]);

%%%%% Certainty Equivalent for risk-adverse activities %%%%%%%
for ia=1:Nagents
    iown=find(OWN==ia);
    iprodland=iown(~ismember(iown,DWELL));
    subtc=travelcost(:,:,ia);
    [subrow,subcol]=ind2sub([NLENGTH NWIDTH],iprodland);
    for ic=1:length(iprodland)
        avgexptyield(1:Nuse-Nuse_nonprod,1)=mean(EXPTPROD(subrow(ic),...
            subcol(ic),1:Nuse-Nuse_nonprod,TSTART+1),4);
        avgexptprod=(avgexptyield.*WTAPRICE(ia,1:Nuse-Nuse_nonprod,TSTART+1)');
        AVGexptyield(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgexptyield;
        AVGexptprod(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgexptprod;        
    end
    
    for iu=1:Nuse-Nuse_nonprod
        subexptay=AVGexptyield(:,:,iu);
        subexptap=AVGexptprod(:,:,iu);
        meanexptyield(iu)=mean(subexptay(iprodland));
        margmeanexptyield(iu)=meanexptyield(iu)./(iulaborcost(iu)+mean(subtc(iprodland)));
        meanexptprod(iu)=mean(subexptap(iprodland));
        FARMWAGE(ia,iu,TSTART)=max(meanexptprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt),0);
        margmeanexptprod(iu)=(meanexptprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt))/(iulaborcost(iu)+mean(subtc(iprodland)));
    end
    ifindexptsublab(ia,TSTART)=find(margmeanexptyield == max(margmeanexptyield));
    ifindexptmktlab(ia,TSTART)=find(FARMWAGE(ia,1:Nuse-Nuse_nonprod,TSTART) == ...
        max(FARMWAGE(ia,1:Nuse-Nuse_nonprod,TSTART)),1,'first');
end

%%% Remove agents that cannot subsist on land
iremove=find(SUBLABOR(:,TSTART) > Ltot);
OWN(ismember(OWN,iremove))=0;

subLU=LU(:,:,TSTART);
icropsint=find(subLU==1);
icropsext=find(subLU==2);
ipasture=find(subLU==3);
itrees=find(subLU==4);
ifallow=find(subLU==5);
idwell=find(subLU==6);
LU(:,:,TSTART+1)=subLU;

LUpct(1:Nuse,TSTART)=([length(icropsint); length(icropsext); length(ipasture); ...
    length(itrees); length(ifallow); length(idwell)])./(NLENGTH*NWIDTH);
%%

for t=TSTART+1:TMAX
    %%
    iNagents=unique(OWN);
    iNagents=iNagents(iNagents~=0);

    subLU=LU(:,:,t);
    icropsint=find(subLU==1);
    icropsext=find(subLU==2);
    ipasture=find(subLU==3);
    itrees=find(subLU==4);
    ifallow=find(subLU==5);
    idwell=find(subLU==6);
    LUpct(1:Nuse,t)=([length(icropsint); length(icropsext); length(ipasture); ...
    length(itrees); length(ifallow); length(idwell)])./(NLENGTH*NWIDTH);
    ALTSET(:,:,Nuse+1)=LU(:,:,t);
    %%%%%%%% Biodynamics of productivity %%%%%%%%%%
    subsoil=zeros(size(LAND));
    
    %<><><><><><><><><><><><>   No Degradation   <><><><><><><><><><><><><>
    BASEPROD(:,:,:,t)=BASEPROD(:,:,:,t-1);
    PROD(:,:,1,t)=prodvar_all(1,t).*BASEPROD(:,:,1,t)+BASEPROD(:,:,1,t);
    PROD(:,:,4,t)=prodvar_all(4,t)*BASEPROD(:,:,4,t)+BASEPROD(:,:,4,t);
    PROD(:,:,5,t)=prodvar_all(5,t)*BASEPROD(:,:,5,t)+BASEPROD(:,:,5,t);
    PROD(:,:,6,t)=prodvar_all(6,t)*BASEPROD(:,:,6,t)+BASEPROD(:,:,6,t);
    %<><><><><><><><><>   Degradation due to use   <><><><><><><><><><><><>
    subcrop=BASEPROD(:,:,2,t-1);
    subbasecrop=BASEPROD(:,:,2,1);
    subcrop(icropsext)=max(subcrop(icropsext)-DEGRAD(2)*subcrop(icropsext),0);
    subcrop(ifallow)=min([subbasecrop(ifallow) abs(subcrop(ifallow)+REGEN(2)*subcrop(ifallow))],[],2);
    BASEPROD(:,:,2,t)=subcrop;
    PROD(:,:,2,t)=prodvar_all(2,t)*BASEPROD(:,:,2,t)+BASEPROD(:,:,2,t);

    subpasture=BASEPROD(:,:,3,t-1);
    subbasepas=BASEPROD(:,:,3,1);
    subpasture(ipasture)=max(subpasture(ipasture)-DEGRAD(3)*subpasture(ipasture),0);
    subpasture(ifallow)=min([subbasepas(ifallow) abs(subpasture(ifallow)+REGEN(3)*...
        subpasture(ifallow))],[],2);
    BASEPROD(:,:,3,t)=subpasture;
    PROD(:,:,3,t)=prodvar_all(3,t)*BASEPROD(:,:,3,t)+BASEPROD(:,:,3,t);
    transcost=laborcosts;
    transcost(transcost == 1000)=0;
    transcost=transcost+inputcosts;
    % Only productive uses
    for iu=1:Nuse-Nuse_nonprod
        ltavgscape(:,:,iu)=mean(PROD(:,:,iu,1:t),4);
    end
    
        %%%%% Certainty Equivalent for risk-adverse activities %%%%%%%
    
    for ia=1:length(iNagents)
        iown=find(OWN == iNagents(ia));
        iprodland=iown(~ismember(iown,DWELL));
        sublus=LU(:,:,t);
        subtc=travelcost(:,:,iNagents(ia));
        [subrow,subcol]=ind2sub([NLENGTH NWIDTH],iprodland);
        for ic=1:length(iprodland)
            % Observed yields
            avgyield(1:Nuse-Nuse_nonprod,1)=mean(PROD(subrow(ic),subcol(ic),...
                1:Nuse-Nuse_nonprod,1:t),4);
            avgprod=(avgyield.*mean(Price(1:Nuse-Nuse_nonprod,1:t),2));
            AVGyield(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgyield;
            AVGprod(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgprod;           
            
            avgexptyield(1:Nuse-Nuse_nonprod,1)=mean(EXPTPROD(subrow(ic),...
                subcol(ic),1:Nuse-Nuse_nonprod,TSTART+1:t),4);
            avgexptprod=(avgexptyield.*mean(WTAPRICE(iNagents(ia),1:Nuse-Nuse_nonprod,TSTART+1:t),3)');
            AVGexptyield(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgexptyield;
            AVGexptprod(subrow(ic),subcol(ic),1:Nuse-Nuse_nonprod)=avgexptprod;
        end
        
        for iu=1:Nuse-Nuse_nonprod
            subexptay=AVGexptyield(:,:,iu);
            subexptap=AVGexptprod(:,:,iu);
            meanexptyield(iu)=mean(subexptay(iprodland));
            margmeanexptyield(iu)=meanexptyield(iu)./(iulaborcost(iu)+mean(subtc(iprodland)));
            meanexptprod(iu)=mean(subexptap(iprodland));
            FARMWAGE(iNagents(ia),iu,t)=max((1-deltapref)*FARMWAGE(iNagents(ia),iu,t-1)+...
            deltapref*(meanexptprod(iu)-farmcosts*(iuinputcost(iu)+time2mkt)),0);
            margmeanexptprod(iu)=(meanexptprod(iu)-farmcosts*(iuinputcost(iu)+...
                time2mkt))/(iulaborcost(iu)+mean(subtc(iprodland)));
            
            subay=AVGyield(:,:,iu);
            subap=AVGprod(:,:,iu);
            meanyield(iu)=mean(subay(iprodland));
            margmeanyield(iu)=meanyield(iu)./(iulaborcost(iu)+mean(subtc(iprodland)));
            meanprod(iu)=mean(subap(iprodland));
            margmeanprod(iu)=(meanprod(iu)-farmcosts*(iuinputcost(iu)+...
                time2mkt))/(iulaborcost(iu)+mean(subtc(iprodland)));
            
            submaxp=PROD(:,:,iu,t);
            meanmaxyield(iu)=mean(submaxp(iprodland));
            margmeanmaxyield(iu)=meanmaxyield(iu)./(iulaborcost(iu)+mean(subtc(iprodland)));
            meanmaxprod(iu)=mean(submaxp(iprodland))*Price(iu,t);
            FARMWAGE_opt(iNagents(ia),iu,t)=max(meanmaxprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt),0);
            margmeanmaxprod(iu)=(meanmaxprod(iu)-farmcosts*(iuinputcost(iu)+...
            time2mkt))/(iulaborcost(iu)+mean(subtc(iprodland)));

            meanmktchk(iu)=meanprod(iu)-farmcosts*(iuinputcost(iu)+time2mkt);
            meanexptmktchk(iu)=meanexptprod(iu)-farmcosts*(iuinputcost(iu)+time2mkt);
            maxmktchk(iu)=meanmaxprod(iu)-farmcosts*(iuinputcost(iu)+time2mkt);
        end
        ifindexptsublab(iNagents(ia),t)=find(margmeanexptyield == max(margmeanexptyield));
        ifindexptmktlab(iNagents(ia),t)=find(FARMWAGE(iNagents(ia),1:Nuse-Nuse_nonprod,t) == ...
            max(FARMWAGE(iNagents(ia),1:Nuse-Nuse_nonprod,t)),1,'first');
        ifindmaxmktlab(iNagents(ia),t)=find(FARMWAGE_opt(iNagents(ia),1:Nuse-Nuse_nonprod,t) == ...
            max(FARMWAGE_opt(iNagents(ia),1:Nuse-Nuse_nonprod,t)),1,'first');
        ifindmaxsublab(iNagents(ia),t)=find(margmeanmaxyield == max(margmeanmaxyield));
        
        foodprice(iNagents(ia),t)=(1-CE(iNagents(ia)))*((2-MKTACC)*farmcosts*...
            (iuinputcost(ifindexptsublab(iNagents(ia),t))+time2mkt));
        shadowprice(iNagents(ia),t)=0.5*farmcosts*...
            (iuinputcost(ifindexptsublab(iNagents(ia),t))+time2mkt);
        
        foodprice_opt(iNagents(ia),t)=(2-MKTACC)*farmcosts*(iuinputcost(ifindmaxsublab(iNagents(ia),t))+time2mkt);
        shadowprice_opt(iNagents(ia),t)=farmcosts*(iuinputcost(ifindmaxsublab(iNagents(ia),t))+time2mkt);        
        NFARMWAGE(iNagents(ia),t)=max((minwage-nonfarmcosts)*...
            (iulaborcost(ifindexptmktlab(iNagents(ia),t))+mean(subtc(iprodland))),0);
        NFARMWAGE_opt(iNagents(ia),t)=max((minwage-nonfarmcosts)*...
            (iulaborcost(ifindmaxmktlab(iNagents(ia),t))+mean(subtc(iprodland))),0);

        if shadowprice(iNagents(ia),t) > foodprice(iNagents(ia),t) && STORE(iNagents(ia),2,t) > 0
            efood(iNagents(ia),t)=foodprice(iNagents(ia),t)/...
                shadowprice(iNagents(ia),t);
            SUBLABOR(iNagents(ia),t)=max(min(SUBLABOR(iNagents(ia),t-1)+...
                ceil(-foodbalance(iNagents(ia),t-1)/margmeanexptyield(ifindexptsublab(iNagents(ia),t))),...
                SUBLABOR(iNagents(ia),TSTART)*efood(iNagents(ia),t)),0);
        else
            SUBLABOR(iNagents(ia),t)=max(min(SUBLABOR(iNagents(ia),t-1)+...
                ceil(-foodbalance(iNagents(ia),t-1)/margmeanexptyield(ifindexptsublab(iNagents(ia),t))),...
                Ltot(iNagents(ia))),0);
        end
        
        inkindwage(iNagents(ia),t)=farmcosts*iuinputcost(ifindexptsublab(iNagents(ia),t))*...
            (SUBLABOR(iNagents(ia),t)/iulaborcost(ifindexptsublab(iNagents(ia),t)));
        
        if SUBLABOR(iNagents(ia),t) > Ltot(iNagents(ia))
            sprintf('Need to remove agent')
        elseif SUBLABOR(iNagents(ia),t) < 0
            sprintf('Need to remove agent')
        end
        
        A(iNagents(ia),t)=(1+MKTINF)*FARMWAGE(iNagents(ia),ifindexptmktlab(iNagents(ia),t),t)*...
            ceil(SUBREQ(iNagents(ia))/margmeanexptyield(ifindexptsublab(iNagents(ia),t)));
        farmincreq(iNagents(ia),t)=farmcosts*((Farmlabor(iNagents(ia),2,t-1)/...
            iulaborcost(ifindexptmktlab(iNagents(ia),t)))*iuinputcost(ifindexptmktlab(iNagents(ia),t))+...
            (SUBLABOR(iNagents(ia),t)/iulaborcost(ifindexptsublab(iNagents(ia),t)))*...
            iuinputcost(ifindexptsublab(iNagents(ia),t)));
        if STORE(iNagents(ia),1,t) > 0
            foodincreq(iNagents(ia),t)=0;
        else
            foodincreq(iNagents(ia),t)=WTAPRICE(iNagents(ia),ifindexptsublab(iNagents(ia),t),t).*...
                abs(foodbalance(iNagents(ia),t-1));
        end
        INCREQ(iNagents(ia),t)=farmincreq(iNagents(ia),t)+foodincreq(iNagents(ia),t)+...
            A(iNagents(ia),t);
        
        %%%%%%%%%%%%%%%%%%%%%%%  Optimal Routine  %%%%%%%%%%%%%%%%%%%%%%%%%
        if shadowprice_opt(iNagents(ia),t) > foodprice_opt(iNagents(ia),t)
            efood_opt(iNagents(ia),t)=foodprice_opt(iNagents(ia),t)/...
                shadowprice_opt(iNagents(ia),t);
            SUBLABOR_opt(iNagents(ia),t)=max(min(SUBLABOR_opt(iNagents(ia),t-1)+...
                ceil(-foodbalance(iNagents(ia),t-1)/margmeanmaxyield(ifindmaxsublab(iNagents(ia),t))),...
                SUBLABOR_opt(iNagents(ia),TSTART)*efood_opt(iNagents(ia),t)),0);
        else
            SUBLABOR_opt(iNagents(ia),t)=max(min(SUBLABOR_opt(iNagents(ia),t-1)+...
                ceil(-foodbalance(iNagents(ia),t-1)/margmeanmaxyield(ifindmaxsublab(iNagents(ia),t))),...
                Ltot(iNagents(ia))),0);
        end
        
        inkindwage_opt(iNagents(ia),t)=farmcosts*iuinputcost(ifindmaxsublab(iNagents(ia),t))*...
            (SUBLABOR_opt(iNagents(ia),t)/iulaborcost(ifindmaxsublab(iNagents(ia),t)));
        
        A_opt(iNagents(ia),t)=(1+MKTINF)*FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t)*...
            ceil(SUBREQ(iNagents(ia))/margmeanmaxyield(ifindmaxsublab(iNagents(ia),t)));
        farmincreq_opt(iNagents(ia),t)=farmcosts*((Farmlabor_opt(iNagents(ia),2,t-1)/...
            iulaborcost(ifindmaxmktlab(iNagents(ia),t)))*iuinputcost(ifindmaxmktlab(iNagents(ia),t))+...
            (SUBLABOR_opt(iNagents(ia),t)/iulaborcost(ifindmaxsublab(iNagents(ia),t)))*...
            iuinputcost(ifindmaxsublab(iNagents(ia),t)));
        if STORE(iNagents(ia),1,t) > 0
            foodincreq_opt(iNagents(ia),t)=0;
        else
            foodincreq_opt(iNagents(ia),t)=Price(ifindmaxsublab(iNagents(ia),t),t).*...
                abs(foodbalance(iNagents(ia),t-1));
        end
        INCREQ_opt(iNagents(ia),t)=farmincreq_opt(iNagents(ia),t)+foodincreq_opt(iNagents(ia),t)+...
            A_opt(iNagents(ia),t);

        MARGPROD(iNagents(ia),1,t)=margmeanyield(ifindexptsublab(iNagents(ia),t));
        MARGPROD(iNagents(ia),2,t)=margmeanprod(ifindexptmktlab(iNagents(ia),t)); 
        
        %%%%%%%%%%%%%%%%%%%%%%   Labor Allocation   %%%%%%%%%%%%%%%%%%%%%%%
        
        CEneut(iNagents(ia),t)=0.5*FARMWAGE(ia,ifindexptmktlab(iNagents(ia),t),t);
        CEra(iNagents(ia),t)=CE(iNagents(ia))*NFARMWAGE(iNagents(ia),t);
        
        if FARMWAGE(iNagents(ia),ifindexptmktlab(iNagents(ia),t),t) == 0 && NFARMWAGE(iNagents(ia),t) == 0
            pctfarminc(iNagents(ia),t)=0;
            pctofffarminc(iNagents(ia),t)=0;
        else
            pctfarminc(iNagents(ia),t)=CEneut(iNagents(ia),t)/...
                (CEneut(iNagents(ia),t)+CEra(iNagents(ia),t));
            pctofffarminc(iNagents(ia),t)=1-pctfarminc(iNagents(ia),t);
        end
        
        if NFARMWAGE(iNagents(ia),t) == 0 && FARMWAGE(iNagents(ia),ifindexptmktlab(iNagents(ia),t),t) == 0
            INCLABOR(iNagents(ia),t)=0;
        elseif NFARMWAGE(iNagents(ia),t) == 0
            INCLABOR(iNagents(ia),t)=ceil((pctfarminc(iNagents(ia),t)*...
                INCREQ(iNagents(ia),t))/FARMWAGE(iNagents(ia),ifindexptmktlab(iNagents(ia),t),t));
        elseif FARMWAGE(iNagents(ia),ifindexptmktlab(iNagents(ia),t),t) == 0
            INCLABOR(iNagents(ia),t)=ceil((pctofffarminc(iNagents(ia),t)*...
                INCREQ(iNagents(ia),t))/NFARMWAGE(iNagents(ia),t));
        else
            INCLABOR(iNagents(ia),t)=ceil((pctofffarminc(iNagents(ia),t)*...
                INCREQ(iNagents(ia),t))/NFARMWAGE(iNagents(ia),t)+(pctfarminc(iNagents(ia),t)*...
                INCREQ(iNagents(ia),t))/FARMWAGE(iNagents(ia),ifindexptmktlab(iNagents(ia),t),t));
        end

        %Allocation of home vs work time
        sigmahome(iNagents(ia),t)=(1-MKTINF)*max(min((1+foodbalance(iNagents(ia),t-1)/...
            SUBREQ(iNagents(ia))),2),0)+MKTINF*max(min(STORE(iNagents(ia),2,t)/...
            INCREQ(iNagents(ia),t),2),0);
        laborprefs(iNagents(ia),t)=max(min((1-deltapref)*laborprefs(iNagents(ia),t-1)+...
            deltapref*(laborprefs(iNagents(ia),t-1)*sigmahome(iNagents(ia),t)),...
            1-(SUBLABOR(iNagents(ia),t)+INCLABOR(iNagents(ia),t))/...
            Ltot(iNagents(ia))),0);
        Labor(iNagents(ia),1,t)=ceil(laborprefs(iNagents(ia),t)*Ltot(iNagents(ia)));
        
        %%%%%%%%%%%% Simpler Alternative %%%%%%%%%%%%
        Labor(iNagents(ia),2,t)=max(min(SUBLABOR(iNagents(ia),t)+...
            ceil(INCLABOR(iNagents(ia),t)*pctfarminc(iNagents(ia),t)),...
            Ltot(iNagents(ia))-Labor(iNagents(ia),1,t)),SUBLABOR(iNagents(ia),t));
            
        Labor(iNagents(ia),3,t)=max(min(ceil(INCLABOR(iNagents(ia),t)*...
            pctofffarminc(iNagents(ia),t)),Ltot(iNagents(ia))-...
            sum(Labor(iNagents(ia),1:2,t),2)),0);
        
        Labor(iNagents(ia),1,t)=Ltot(iNagents(ia))-sum(Labor(iNagents(ia),2:3,t));
        
        %Allocation of sub vs market production
        Farmlabor(iNagents(ia),1,t)=max(min(ceil((1-deltapref)*SUBLABOR(iNagents(ia),t-1)+...
            deltapref*SUBLABOR(iNagents(ia),t)),Labor(iNagents(ia),2,t)),SUBLABOR(iNagents(ia),t));
        Farmlabor(iNagents(ia),2,t)=max(min(ceil((1-deltapref)*Farmlabor(iNagents(ia),2,t-1)+...
            deltapref*(Labor(iNagents(ia),2,t)-Farmlabor(iNagents(ia),1,t))),...
            Labor(iNagents(ia),2,t)-Farmlabor(iNagents(ia),1,t)),0);
        
        %%%%%%%%%%%%%%%%%%%%   Optimal Labor Allocation   %%%%%%%%%%%%%%%%%
        CEneut_opt(iNagents(ia),t)=FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t);
        CEra_opt(iNagents(ia),t)=NFARMWAGE_opt(iNagents(ia),t);
        
        if FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t) == 0 && NFARMWAGE_opt(iNagents(ia),t) == 0
            pctfarminc_opt(iNagents(ia),t)=0;
            pctofffarminc_opt(iNagents(ia),t)=0;
        else
            pctfarminc_opt(iNagents(ia),t)=CEneut_opt(iNagents(ia),t)/...
                (CEneut_opt(iNagents(ia),t)+CEra_opt(iNagents(ia),t));
            pctofffarminc_opt(iNagents(ia),t)=1-pctfarminc_opt(iNagents(ia),t);
        end
        
        if NFARMWAGE_opt(iNagents(ia),t) == 0 && FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t) == 0
            INCLABOR_opt(iNagents(ia),t)=0;
        elseif NFARMWAGE_opt(iNagents(ia),t) == 0
            INCLABOR_opt(iNagents(ia),t)=ceil((pctfarminc_opt(iNagents(ia),t)*...
                INCREQ_opt(iNagents(ia),t))/FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t));
        elseif FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t) == 0
            INCLABOR_opt(iNagents(ia),t)=ceil((pctofffarminc_opt(iNagents(ia),t)*...
                INCREQ_opt(iNagents(ia),t))/NFARMWAGE_opt(iNagents(ia),t));
        else
            INCLABOR_opt(iNagents(ia),t)=ceil((pctofffarminc_opt(iNagents(ia),t)*...
                INCREQ_opt(iNagents(ia),t))/NFARMWAGE_opt(iNagents(ia),t)+...
                (pctfarminc_opt(iNagents(ia),t)*INCREQ_opt(iNagents(ia),t))/...
                FARMWAGE_opt(iNagents(ia),ifindmaxmktlab(iNagents(ia),t),t));
        end
        
        
        %Allocation of home vs work time
        sigmahome_opt(iNagents(ia),t)=(1-MKTINF)*max(min((1+foodbalance(iNagents(ia),t-1)/...
            SUBREQ(iNagents(ia))),2),0)+MKTINF*max(min(STORE(iNagents(ia),2,t)/...
            INCREQ_opt(iNagents(ia),t),2),0);
        laborprefs_opt(iNagents(ia),t)=max(min(laborprefs_opt(iNagents(ia),t-1)*...
            sigmahome_opt(iNagents(ia),t),1-(SUBLABOR_opt(iNagents(ia),t)+...
            INCLABOR_opt(iNagents(ia),t))/Ltot(iNagents(ia))),0);
        Labor_opt(iNagents(ia),1,t)=laborprefs_opt(iNagents(ia),t)*Ltot(iNagents(ia));
        
        %%%%%%%%%%%% Simpler Alternative %%%%%%%%%%%%
        Labor_opt(iNagents(ia),2,t)=max(min(SUBLABOR_opt(iNagents(ia),t)+ceil((Ltot(iNagents(ia))-...
            Labor_opt(iNagents(ia),1,t))*pctfarminc_opt(iNagents(ia),t)),Ltot(iNagents(ia))-...
            Labor_opt(iNagents(ia),1,t)),SUBLABOR_opt(iNagents(ia),t));
            
        Labor_opt(iNagents(ia),3,t)=max(min(ceil((Ltot(iNagents(ia))-Labor_opt(iNagents(ia),1,t))*...
            pctofffarminc_opt(iNagents(ia),t)),Ltot(iNagents(ia))-...
            sum(Labor_opt(iNagents(ia),1:2,t),2)),0);
        Labor_opt(iNagents(ia),1,t)=Ltot(iNagents(ia))-sum(Labor_opt(iNagents(ia),2:3,t));
        %Allocation of sub vs market production
        Farmlabor_opt(iNagents(ia),1,t)=SUBLABOR_opt(iNagents(ia),t);
        Farmlabor_opt(iNagents(ia),2,t)=max(min(ceil(Labor_opt(iNagents(ia),2,t)-...
            Farmlabor_opt(iNagents(ia),1,t)),Labor_opt(iNagents(ia),2,t)-...
            Farmlabor_opt(iNagents(ia),1,t)),0);
    end
    
    %%% <><><><><><><><> Expected Marginal Production <><><><><><><><><><>
    
    for iu=1:Nuse-Nuse_nonprod
        exptprod=EXPTPROD(:,:,iu,t);
        subprodsub=subscape(:,:,iu,t);
        subprodmkt=mktscape(:,:,iu,t);
        margsub=margsubprod(:,:,iu,t);
        margmkt=margmktprod(:,:,iu,t);
        indcost=sub2ind(size(transmat),subLU,ALTSET(:,:,iu));
        sublabor=laborcosts(indcost);
        subinputcost=inputcosts(indcost);
        hiredlabor=(minwage+nonfarmcosts)*laborcosts(indcost);
        subtreeprod=EXPTPROD(:,:,3,t);
        
        %%%%%%%% Only Production %%%%%%%%%%
        for ia=1:length(iNagents)
            iown=find(OWN == iNagents(ia));
            iprodland=iown(~ismember(iown,DWELL));
            sublu(1:length(iown))=subLU(iown);
            subtc=travelcost(:,:,iNagents(ia));
            if iu == 1 || iu == 2 || iu == 3
                subprodsub(iprodland)=exptprod(iprodland);
                subprodmkt(iprodland)=WTAPRICE(iNagents(ia),iu,t)*exptprod(iprodland);
                margsub(iprodland)=exptprod(iprodland)./(sublabor(iprodland)+subtc(iprodland));
                margmkt(iprodland)=(WTAPRICE(iNagents(ia),iu,t)*exptprod(iprodland)-...
                    (farmcosts*(subinputcost(iprodland)+time2mkt)))./...
                    (sublabor(iprodland)+subtc(iprodland));
            elseif iu == 4
                subprodmkt(iprodland)=WTAPRICE(iNagents(ia),iu,t)*exptprod(iprodland);
                margmkt(iprodland)=(WTAPRICE(iNagents(ia),iu,t)*exptprod(iprodland)-...
                    (farmcosts*(subinputcost(iprodland)+time2mkt)))./...
                    (sublabor(iprodland)+subtc(iprodland));
            elseif iu == 5
                if isempty(find(ismember(itrees,iprodland)==1,1))==0
                    istree=ismember(itrees,iprodland);
                    iharvest=find(timeinuse(itrees(istree)) >= 10);
                    igrow=find(timeinuse(itrees(istree)) < 10);
                    subprodmkt(iprodland(iharvest))=(WTAPRICE(iNagents(ia),4,t)*subtreeprod(iprodland(iharvest)))./...
                        (1+(timeinuse(iprodland(iharvest))-10)*0.05);
                    margmkt(iprodland(iharvest))=(WTAPRICE(iNagents(ia),4,t)*subtreeprod(iprodland(iharvest)))./...
                        (1+(timeinuse(iprodland(iharvest))-10)*0.05)-...
                        (farmcosts*(time2mkt+subtc(iprodland(iharvest))));
                    subprodmkt(iprodland(igrow))=(WTAPRICE(iNagents(ia),4,t)*subtreeprod(iprodland(igrow)))./...
                        (1+(10-timeinuse(iprodland(igrow)))*0.15);
                    margmkt(iprodland(igrow))=subprodmkt(iprodland(igrow));
                end
            elseif iu == 6
                idwell=ismember(iown,DWELL(iNagents(ia)));
                subprodsub(iown(idwell))=meanPROD(6);
                margsub(iown(idwell))=meanPROD(6);
            end
        end
        
        subscape(:,:,iu,t)=subprodsub;
        mktscape(:,:,iu,t)=subprodmkt;
        margsubprod(:,:,iu,t)=margsub;
        margmktprod(:,:,iu,t)=margmkt;
        submargmean(iu,t)=mean(mean(margsub));
        mktmargmean(iu,t)=mean(mean(margmkt));
    end
    
    %%%%% Optimal Routine %%%
    for iu=1:Nuse-Nuse_nonprod
        optprod=PROD(:,:,iu,t);
        subprodsub_opt=subscape_opt(:,:,iu,t);
        subprodmkt_opt=mktscape_opt(:,:,iu,t);
        margsub_opt=margsubprod_opt(:,:,iu,t);
        margmkt_opt=margmktprod_opt(:,:,iu,t);
        indcost=sub2ind(size(transmat),subLU,ALTSET(:,:,iu));
        sublabor=laborcosts(indcost);
        subinputcost=inputcosts(indcost);
        hiredlabor=(minwage+nonfarmcosts)*laborcosts(indcost);
        subtreeprod_opt=PROD(:,:,3,t);
        
        %%%%%%%% Only Production %%%%%%%%%%
        for ia=1:length(iNagents)
            iown=find(OWN == iNagents(ia));
            iprodland=iown(~ismember(iown,DWELL));
            sublu(1:length(iown))=subLU(iown);
            subtc=travelcost(:,:,iNagents(ia));
            if iu == 1 || iu == 2 || iu == 3
                subprodsub_opt(iprodland)=optprod(iprodland);
                subprodmkt_opt(iprodland)=Price(iu,t)*optprod(iprodland);
                margsub_opt(iprodland)=optprod(iprodland)./(sublabor(iprodland)+subtc(iprodland));
                margmkt_opt(iprodland)=(Price(iu,t)*optprod(iprodland)-...
                    (farmcosts*(subinputcost(iprodland)+time2mkt)))./...
                    (sublabor(iprodland)+subtc(iprodland));
            elseif iu == 4
                subprodmkt_opt(iprodland)=Price(iu,t)*optprod(iprodland);
                margmkt_opt(iprodland)=(Price(iu,t)*optprod(iprodland)-...
                    (farmcosts*(subinputcost(iprodland)+time2mkt)))./...
                    (sublabor(iprodland)+subtc(iprodland));
            elseif iu == 5
                if isempty(find(ismember(itrees,iprodland)==1,1))==0
                    istree=ismember(itrees,iprodland);
                    iharvest=find(timeinuse(itrees(istree)) >= 10);
                    igrow=find(timeinuse(itrees(istree)) < 10);
                    subprodmkt_opt(iprodland(iharvest))=(Price(4,t)*subtreeprod_opt(iprodland(iharvest)))./...
                        (1+(timeinuse(iprodland(iharvest))-10)*0.05);
                    margmkt_opt(iprodland(iharvest))=(Price(4,t)*subtreeprod_opt(iprodland(iharvest)))./...
                        (1+(timeinuse(iprodland(iharvest))-10)*0.05)-...
                        (farmcosts*(time2mkt+subtc(iprodland(iharvest))));
                    subprodmkt_opt(iprodland(igrow))=(Price(4,t)*subtreeprod_opt(iprodland(igrow)))./...
                        (1+(10-timeinuse(iprodland(igrow)))*0.15);
                    margmkt_opt(iprodland(igrow))=subprodmkt_opt(iprodland(igrow));
                end
            elseif iu == 6
                idwell=ismember(iown,DWELL(iNagents(ia)));
                subprodsub_opt(iown(idwell))=meanPROD(6);
                margsub_opt(iown(idwell))=meanPROD(6);
            end
        end
        
        subscape_opt(:,:,iu,t)=subprodsub_opt;
        mktscape_opt(:,:,iu,t)=subprodmkt_opt;
        margsubprod_opt(:,:,iu,t)=margsub_opt;
        margmktprod_opt(:,:,iu,t)=margmkt_opt;
    end
 
    subaltset=ALTSET(:,:,Nuse+1);
    %%%%%% Land Use Selection and Allocation of Labor %%%%%%
    activeuse=zeros([],6);
    activeprod=zeros([],5);
    for a=1:length(iNagents)
        iparcel=find(OWN == iNagents(a));
        paraltset=reshape(subaltset(iparcel),PARCEL,PARCEL);
        [row,col]=ind2sub(size(LAND),iparcel);
        irow=unique(row);
        icol=unique(col);
        globalref=globalind(irow,icol);
        subtc=travelcost(:,:,iNagents(a));

        pmargsub=margsubprod(irow,icol,:,t);
        pmargmkt=margmktprod(irow,icol,:,t);
        
        %%% Land Use Preferences %%%
        for iu=1:Nuse
            luprefs(iNagents(a),iu,t)=1;
            EUsub(irow,icol,iu,t)=luprefs(iNagents(a),iu,t).*pmargsub(:,:,iu);
            EUmkt(irow,icol,iu,t)=luprefs(iNagents(a),iu,t).*pmargmkt(:,:,iu);
        end
        %%% Expected Utility for Subsistence%%%
        [maxmargsub,margsubind]=max(EUsub(irow,icol,1:Nuse-Nuse_nonprod,t),[],3);
        
        sublabind=sub2ind(size(transmat),paraltset,margsubind);
        margsublabtime=laborcosts(sublabind)+reshape(subtc(iparcel),PARCEL,PARCEL);
        
        maxsubprodlist=[reshape(maxmargsub,NLENGTH,1) ...
            reshape(margsubind,NLENGTH,1) ...
            reshape(margsublabtime,NLENGTH,1)...
            reshape(subtc(irow,icol),NLENGTH,1) ...
            reshape(farmcosts*inputcosts(sublabind),NLENGTH,1) ...
            (1:NLENGTH)'];
        subprodrank=sortrows(maxsubprodlist,-1);
        
        ipossubcut=find(subprodrank(:,1) > 0,1,'last');
        
        isublabcut=find(cumsum(subprodrank(:,3)) <= Farmlabor(iNagents(a),1,t),1,'last');
        isubmoncut=length(subprodrank(:,5));
        if isempty(find(ipossubcut,1)) == 1
            isubcut=0;
        elseif isempty(find(isublabcut,1)) == 1 || isempty(find(isubmoncut,1)) == 1
            isubcut=0;
        elseif isubmoncut >= isublabcut
            isubcut=min(min(isublabcut,ipossubcut),length(iparcel));
        else
            newset=zeros([],length(subprodrank(1,:)));
            subset=subprodrank(isubmoncut+1:isublabcut,:);
            inocost=find(subset(:,5) == 0);
            icost=find(subset(:,5) > 0);
            newset(1:length(inocost),:)=subset(inocost,:);
            newset(length(newset(:,1))+1:length(newset(:,1))+length(icost),:)=subset(icost,:);
            subprodrank(isubmoncut+1:isublabcut,:)=newset;
            isubcut=min(min(isubmoncut+length(inocost),ipossubcut),length(iparcel));
        end
    
        
        activeuse(length(activeuse(:,1))+1:length(activeuse(:,1))+length(1:isubcut),:)=...
            [subprodrank(1:isubcut,1:5)  globalref(subprodrank(1:isubcut,6))];
       
        [icutrow,icutcol]=ind2sub(size(LAND),globalref(subprodrank(1:isubcut,6)));
        activeprod(length(activeprod(:,1))+1:length(activeprod(:,1))+length(1:isubcut),:)=...
            [icutrow icutcol subprodrank(1:isubcut,2) iNagents(a)*ones(isubcut,1) ones(isubcut,1)];
        %[row col use agentid sub(1)/mkt(2)]
        
        % Remove from market production
        for iu=1:Nuse-Nuse_nonprod
            cutprod=EUmkt(irow,icol,iu,t);
            cutprod(subprodrank(1:isubcut,6))=-1000;
            EUmkt(irow,icol,iu,t)=cutprod;
        end
        
        %%% Expected Utility for Subsistence%%%
        [maxmargmkt,margmktind]=max(EUmkt(irow,icol,1:Nuse-Nuse_nonprod,t),[],3);
        
        mktlabind=sub2ind(size(transmat),paraltset,margmktind);
        margmktlabtime=laborcosts(mktlabind)+...
            time2mkt*reshape(subtc(iparcel),PARCEL,PARCEL);
        
        maxmktprodlist=[reshape(maxmargmkt,NLENGTH,1) ...
            reshape(margmktind,NLENGTH,1) ...
            reshape(margmktlabtime,NLENGTH,1)...
            reshape(time2mkt*subtc(irow,icol),NLENGTH,1)...
            reshape(farmcosts*(inputcosts(mktlabind)+subtc(mktlabind)*time2mkt),...
            NLENGTH,1) (1:NLENGTH)'];
        mktprodrank=sortrows(maxmktprodlist,-1);
        
        iposmktcut=find(mktprodrank(:,1) > 0,1,'last');
        
        imktlabcut=find(cumsum(mktprodrank(:,3)) <= Farmlabor(iNagents(a),2,t),1,'last');
        imktmoncut=find(cumsum(mktprodrank(:,5)) <= STORE(iNagents(a),2,t)-...
            sum(mktprodrank(1:isubcut,5)),1,'last');
        
        if isempty(find(iposmktcut,1)) == 1
            imktcut=0;
        elseif isempty(find(imktlabcut,1)) == 1 || isempty(find(imktmoncut,1)) == 1
            imktcut=0;
        elseif imktmoncut >= imktlabcut
            imktcut=min(min(imktlabcut,iposmktcut),length(iparcel));
        else
            newset=zeros([],length(mktprodrank(1,:)));
            mktset=mktprodrank(imktmoncut+1:imktlabcut,:);
            inocost=find(mktset(:,5) == 0);
            icost=find(mktset(:,5) > 0);
            newset(1:length(inocost),:)=mktset(inocost,:);
            newset(length(newset(:,1))+1:length(newset(:,1))+length(icost),:)=mktset(icost,:);
            mktprodrank(imktmoncut+1:imktlabcut,:)=newset;
            imktcut=min(min(imktmoncut+length(inocost),iposmktcut),length(iparcel));
        end
        
        activeuse(length(activeuse(:,1))+1:length(activeuse(:,1))+length(1:imktcut),:)=...
            [mktprodrank(1:imktcut,1:5) globalref(mktprodrank(1:imktcut,6))];  
        
        [icutrow,icutcol]=ind2sub(size(LAND),globalref(mktprodrank(1:imktcut,6)));
        activeprod(length(activeprod(:,1))+1:length(activeprod(:,1))+length(1:imktcut),:)=...
            [icutrow icutcol mktprodrank(1:imktcut,2) iNagents(a)*ones(imktcut,1) 2*ones(imktcut,1)];
    end
    
    %%%%% Optimal Routine %%%%%
    activeuse_opt=zeros([],6);
    activeprod_opt=zeros([],5);
    for a=1:length(iNagents)
        iparcel=find(OWN == iNagents(a));
        paraltset=reshape(subaltset(iparcel),PARCEL,PARCEL);
        [row,col]=ind2sub(size(LAND),iparcel);
        irow=unique(row);
        icol=unique(col);
        globalref=globalind(irow,icol);
        subtc=travelcost(:,:,iNagents(a));

        pmargsub_opt=margsubprod_opt(irow,icol,:,t);
        pmargmkt_opt=margmktprod_opt(irow,icol,:,t);
        
        %%% Land Use Preferences %%%
        for iu=1:Nuse
            if iu == 1 || iu == 2 || iu == 3
                luprefs_opt(iNagents(a),iu,t)=max(min(deltapref*((1+(PCTINC_opt(iNagents(a),iu,t-1)-...
                    luprefs(iNagents(a),iu,t-1)))*luprefs(iNagents(a),iu,t-1))+...
                    (1-deltapref)*luprefs(iNagents(a),iu,t-1),1),0);
            else
                luprefs_opt(iNagents(a),iu,t)=luprefs(iNagents(a),iu,t-1);
            end
            EUsub_opt(irow,icol,iu,t)=luprefs_opt(iNagents(a),iu,t).*pmargsub_opt(:,:,iu);
            EUmkt_opt(irow,icol,iu,t)=luprefs_opt(iNagents(a),iu,t).*pmargmkt_opt(:,:,iu);
        end
        %%% Expected Utility for Subsistence%%%
        [maxmargsub_opt,margsubind_opt]=max(EUsub_opt(irow,icol,1:Nuse-Nuse_nonprod,t),[],3);
        
        sublabind_opt=sub2ind(size(transmat),paraltset,margsubind_opt);
        margsublabtime=laborcosts(sublabind_opt)+reshape(subtc(iparcel),PARCEL,PARCEL);
        
        maxsubprodlist_opt=[reshape(maxmargsub_opt,NLENGTH,1) ...
            reshape(margsubind_opt,NLENGTH,1) ...
            reshape(margsublabtime,NLENGTH,1)...
            reshape(subtc(irow,icol),NLENGTH,1) ...
            reshape(farmcosts*inputcosts(sublabind_opt),NLENGTH,1) ...
            (1:NLENGTH)'];
        subprodrank_opt=sortrows(maxsubprodlist_opt,-1);
        
        ipossubcut_opt=find(subprodrank_opt(:,1) > 0,1,'last');
        
        isublabcut_opt=find(cumsum(subprodrank_opt(:,3)) <= Farmlabor_opt(iNagents(a),1,t),1,'last');
        isubmoncut_opt=length(subprodrank_opt(:,5));
        
        if isempty(find(ipossubcut_opt,1)) == 1
            isubcut_opt=0;
        elseif isempty(find(isublabcut_opt,1)) == 1 || isempty(find(isubmoncut_opt,1)) == 1
            isubcut_opt=0;
        elseif isubmoncut_opt >= isublabcut_opt
            isubcut_opt=min(min(isublabcut_opt,ipossubcut_opt),length(iparcel));
        else
            newset_opt=zeros([],length(subprodrank_opt(1,:)));
            subset_opt=subprodrank_opt(isubmoncut_opt+1:isublabcut_opt,:);
            inocost_opt=find(subset_opt(:,5) == 0);
            icost_opt=find(subset_opt(:,5) > 0);
            newset_opt(1:length(inocost_opt),:)=subset_opt(inocost_opt,:);
            newset_opt(length(newset_opt(:,1))+1:length(newset_opt(:,1))+...
                length(icost_opt),:)=subset_opt(icost_opt,:);
            subprodrank_opt(isubmoncut_opt+1:isublabcut_opt,:)=newset_opt;
            isubcut_opt=min(min(isubmoncut_opt+length(inocost_opt),ipossubcut_opt),length(iparcel));
        end
    
        activeuse_opt(length(activeuse_opt(:,1))+1:length(activeuse_opt(:,1))+length(1:isubcut_opt),:)=...
            [subprodrank_opt(1:isubcut_opt,1:5)  globalref(subprodrank_opt(1:isubcut_opt,6))];
       
        [icutrow_opt,icutcol_opt]=ind2sub(size(LAND),globalref(subprodrank_opt(1:isubcut_opt,6)));
        activeprod_opt(length(activeprod_opt(:,1))+1:length(activeprod_opt(:,1))+length(1:isubcut_opt),:)=...
            [icutrow_opt icutcol_opt subprodrank_opt(1:isubcut_opt,2) iNagents(a)*ones(isubcut_opt,1) ones(isubcut_opt,1)];
        %[row col use agentid sub(1)/mkt(2)]
        
        % Remove from market production
        for iu=1:Nuse-Nuse_nonprod
            cutprod_opt=EUmkt_opt(irow,icol,iu,t);
            cutprod_opt(subprodrank_opt(1:isubcut_opt,6))=-1000;
            EUmkt_opt(irow,icol,iu,t)=cutprod_opt;
        end
        
        %%% Expected Utility for Subsistence%%%
        [maxmargmkt_opt,margmktind_opt]=max(EUmkt_opt(irow,icol,1:Nuse-Nuse_nonprod,t),[],3);
        
        mktlabind_opt=sub2ind(size(transmat),paraltset,margmktind_opt);
        margmktlabtime=laborcosts(mktlabind_opt)+time2mkt*reshape(subtc(iparcel),PARCEL,PARCEL);
        
        maxmktprodlist_opt=[reshape(maxmargmkt_opt,NLENGTH,1) ...
            reshape(margmktind_opt,NLENGTH,1) ...
            reshape(margmktlabtime,NLENGTH,1)...
            reshape(time2mkt*subtc(irow,icol),NLENGTH,1)...
            reshape(farmcosts*(inputcosts(mktlabind_opt)+subtc(mktlabind_opt)*time2mkt),...
            NLENGTH,1) (1:NLENGTH)'];
        mktprodrank_opt=sortrows(maxmktprodlist_opt,-1);
        
        iposmktcut_opt=find(mktprodrank_opt(:,1) > 0,1,'last');
        
        imktlabcut_opt=find(cumsum(mktprodrank_opt(:,3)) <= Farmlabor_opt(iNagents(a),2,t),1,'last');
        imktmoncut_opt=find(cumsum(mktprodrank_opt(:,5)) <= STORE_opt(iNagents(a),2,t)-...
            sum(mktprodrank_opt(1:isubcut_opt,5)),1,'last');
        
        if isempty(find(iposmktcut_opt,1)) == 1
            imktcut_opt=0;
        elseif isempty(find(imktlabcut_opt,1)) == 1 || isempty(find(imktmoncut_opt,1)) == 1
            imktcut_opt=0;
        elseif imktmoncut_opt >= imktlabcut_opt
            imktcut_opt=min(min(imktlabcut_opt,iposmktcut_opt),length(iparcel));
        else
            newset_opt=zeros([],length(mktprodrank_opt(1,:)));
            mktset_opt=mktprodrank_opt(imktmoncut_opt+1:imktlabcut_opt,:);
            inocost_opt=find(mktset_opt(:,5) == 0);
            icost_opt=find(mktset_opt(:,5) > 0);
            newset_opt(1:length(inocost_opt),:)=mktset_opt(inocost_opt,:);
            newset_opt(length(newset_opt(:,1))+1:length(newset_opt(:,1))+length(icost_opt),:)=mktset_opt(icost_opt,:);
            mktprodrank_opt(imktmoncut_opt+1:imktlabcut_opt,:)=newset_opt;
            imktcut_opt=min(min(imktmoncut_opt+length(inocost_opt),iposmktcut_opt),length(iparcel));
        end
        
        activeuse_opt(length(activeuse_opt(:,1))+1:length(activeuse_opt(:,1))+length(1:imktcut_opt),:)=...
            [mktprodrank_opt(1:imktcut_opt,1:5) globalref(mktprodrank_opt(1:imktcut_opt,6))];  
        
        [icutrow_opt,icutcol_opt]=ind2sub(size(LAND),globalref(mktprodrank_opt(1:imktcut_opt,6)));
        activeprod_opt(length(activeprod_opt(:,1))+1:length(activeprod_opt(:,1))+length(1:imktcut_opt),:)=...
            [icutrow_opt icutcol_opt mktprodrank_opt(1:imktcut_opt,2) iNagents(a)*ones(imktcut_opt,1) 2*ones(imktcut_opt,1)];
    end
    
    %%%%% Real Production %%%%%
    realprod=zeros(length(activeprod(:,1)),3);
    realprod(:,2)=activeuse(:,3);
    for ip=1:length(activeprod(:,1))
        realprod(ip,3)=PROD(activeprod(ip,1),activeprod(ip,2),activeprod(ip,3),t);
        if activeprod(ip,5) == 1 && activeuse(ip,2) == 1 || activeuse(ip,2) == 2 || activeuse(ip,2) == 3
            realprod(ip,1)=PROD(activeprod(ip,1),activeprod(ip,2),activeprod(ip,3),t);
        elseif activeprod(ip,5) == 2
            realprod(ip,1)=Price(activeprod(ip,3),t)*...
                PROD(activeprod(ip,1),activeprod(ip,2),activeprod(ip,3),t)-...
                farmcosts*(iuinputcost(activeuse(ip,2))+time2mkt);
        end
    end
    realprod(realprod(:,1)==0,3)=0;
    
    %%%%% Optimal Production %%%%%
    realprod_opt=zeros(length(activeprod_opt(:,1)),3);
    realprod_opt(:,2)=activeuse_opt(:,3);
    for ip=1:length(activeprod_opt(:,1))
        realprod_opt(ip,3)=PROD(activeprod_opt(ip,1),activeprod_opt(ip,2),activeprod_opt(ip,3),t);
        if activeprod_opt(ip,5) == 1 && activeuse_opt(ip,2) == 1 || activeuse_opt(ip,2) == 2 || activeuse_opt(ip,2) == 3
            realprod_opt(ip,1)=PROD(activeprod_opt(ip,1),activeprod_opt(ip,2),activeprod_opt(ip,3),t);
        elseif activeprod_opt(ip,5) == 2
            realprod_opt(ip,1)=Price(activeprod_opt(ip,3),t)*...
                PROD(activeprod_opt(ip,1),activeprod_opt(ip,2),activeprod_opt(ip,3),t)-...
                farmcosts*(iuinputcost(activeuse_opt(ip,2))+time2mkt);
        end
    end
    realprod_opt(realprod_opt(:,1)==0,3)=0;
    
    %%%%% Full Income and Consumption %%%%%
    for ia=1:length(iNagents)
        iprod=find(activeprod(:,4) == iNagents(ia));
        isubprod=(activeprod(iprod,5) == 1 & realprod(iprod,1) ~= 0);
        imktprod=(activeprod(iprod,5) == 2 & realprod(iprod,1) ~= 0);
        
        for iu=1:Nuse-Nuse_nonprod
            ilu=find(activeuse(iprod,2) == iu);
            PCTINC(iNagents(ia),iu,t)=sum(realprod(iprod(ilu),3))/...
                sum(realprod(iprod,3));
        end
        grossinc(iNagents(ia),t)=Labor(iNagents(ia),3,t)*minwage+sum(realprod(iprod(imktprod),1));
        if grossinc(iNagents(ia),t) <= 0
            STORE(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t)+inkindwage(iNagents(ia),t)+...
                (Labor(iNagents(ia),3,t)*minwage+sum(realprod(iprod(imktprod),1)))-...
                (Labor(iNagents(ia),3,t)*nonfarmcosts+sum(activeuse(imktprod,5)));
        else
            STORE(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t)+...
                (Labor(iNagents(ia),3,t)*minwage+sum(realprod(iprod(imktprod),1)))-...
                (Labor(iNagents(ia),3,t)*nonfarmcosts+sum(activeuse(imktprod,5)));
        end
        inclevel(iNagents(ia),t)=STORE(iNagents(ia),2,t+1);
        STORE(iNagents(ia),1,t+1)=STORE(iNagents(ia),1,t)+...
            sum(realprod(iprod(isubprod),1))-SUBREQ(iNagents(ia));
        foodbalance(iNagents(ia),t)=STORE(iNagents(ia),1,t)+...
            sum(realprod(iprod(isubprod),1))-SUBREQ(iNagents(ia));
        foodprod(iNagents(ia),t)=sum(realprod(iprod(isubprod),1));
        
        if isempty(find(STORE(iNagents(ia),1,t+1) < 0,1))==0
            STORE(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t+1)+...
                STORE(iNagents(ia),1,t+1)*Price(1,t);
            STORE(iNagents(ia),1,t+1)=0;
        end
        if isempty(find(STORE(iNagents(ia),2,t+1) < 0,1))==0
            sprintf('Negative STORE')
            STORE(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t+1)+...
                inkindparm*SUBREQ(ia)*Price(1,t);
        end
    end
    
    %%%%% Optimal Full Income and Consumption %%%%%
    for ia=1:length(iNagents)
        iprod_opt=find(activeprod_opt(:,4) == iNagents(ia));
        isubprod_opt=(activeprod_opt(iprod_opt,5) == 1 & realprod_opt(iprod_opt,1) ~= 0);
        imktprod_opt=(activeprod_opt(iprod_opt,5) == 2 & realprod_opt(iprod_opt,1) ~= 0);
        
        for iu=1:3
            ilu_opt=find(activeuse_opt(iprod_opt,2) == iu);
            PCTINC_opt(iNagents(ia),iu,t)=sum(realprod_opt(iprod_opt(ilu_opt),3))/...
                sum(realprod_opt(iprod_opt,3));
        end
        grossinc_opt(iNagents(ia),t)=Labor_opt(iNagents(ia),3,t)*minwage+...
            sum(realprod_opt(iprod_opt(imktprod_opt),1));
        
        if grossinc_opt(iNagents(ia),t) <= 0
            STORE_opt(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t)+inkindwage_opt(iNagents(ia),t)+...
                (Labor_opt(iNagents(ia),3,t)*minwage+sum(realprod_opt(iprod_opt(imktprod_opt),1)))-...
                (Labor_opt(iNagents(ia),3,t)*nonfarmcosts+sum(activeuse_opt(imktprod_opt,5)));
        else
            STORE_opt(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t)+...
                (Labor_opt(iNagents(ia),3,t)*minwage+sum(realprod_opt(iprod_opt(imktprod_opt),1)))-...
                (Labor_opt(iNagents(ia),3,t)*nonfarmcosts+sum(activeuse_opt(imktprod_opt,5)));
        end
        
        STORE_opt(iNagents(ia),1,t+1)=STORE(iNagents(ia),1,t)+...
            sum(realprod_opt(iprod_opt(isubprod_opt),1))-SUBREQ(iNagents(ia));
        foodbalance_opt(iNagents(ia),t)=STORE(iNagents(ia),1,t)+...
            sum(realprod_opt(iprod_opt(isubprod_opt),1))-SUBREQ(iNagents(ia));
        
        if isempty(find(STORE_opt(iNagents(ia),1,t+1) < 0,1))==0
            STORE_opt(iNagents(ia),2,t+1)=STORE_opt(iNagents(ia),2,t+1)+...
                STORE_opt(iNagents(ia),1,t+1)*Price(1,t);
            STORE_opt(iNagents(ia),1,t+1)=0;
        end
        if isempty(find(STORE_opt(iNagents(ia),2,t+1) < 0,1))==0
            sprintf('Negative STORE')
            STORE(iNagents(ia),2,t+1)=STORE(iNagents(ia),2,t+1)+...
                inkindparm*SUBREQ(ia)*Price(ifindexptsublab(iNagents(ia),t),t);
        end
    end
    
%%%<><><><><><><><><><>   Update Land-Use   <><><><><><><><><><><><><><><>

    newLU=LU(:,:,t);
    currentLU=newLU;
    newLU(activeuse(:,6))=activeuse(:,2);
    
    produse=zeros(NLENGTH,NWIDTH);
    iactiveuse=(activeuse(:,2) < Nuse-2);
    produse(activeuse(iactiveuse,6))=1;
    iinprod=find(produse==1);
    ioutprod=find(produse==0);
    timeinprod(iinprod)=timeinprod(iinprod)+1;
    timeinprod(ioutprod)=0;
    timeoutprod(ioutprod)=timeoutprod(ioutprod)+1;
    timeoutprod(iinprod)=0;
    
    LU(:,:,t+1)=newLU;
    
    newfallow=LU(:,:,t+1); 
    
    % crops go to fallow after one year non-use
    inoncropuse=(timeoutprod(ioutprod) >= 1 & newfallow(ioutprod)==1);
    newfallow(ioutprod(inoncropuse))=5;
    
    inonextcrop=(timeoutprod(ioutprod) >= 1 & newfallow(ioutprod)==2);
    newfallow(ioutprod(inonextcrop))=5;
    
    % pastures go to fallow after two years non-use
    inonpasture=(newfallow(ioutprod)==3 & timeoutprod(ioutprod) >= 1);
    newfallow(ioutprod(inonpasture))=5;
    
%     inewforest=(newfallow(ioutprod)==5 & timeoutprod(ioutprod) >= 5);
    inewforest=(newfallow(ioutprod)==5 & timeoutprod(ioutprod) >= 10);
    newfallow(ioutprod(inewforest))=4;
    
    LU(:,:,t+1)=newfallow;
    
    isameuse=(LU(:,:,t+1)==LU(:,:,t));
    inewuse=(LU(:,:,t+1)~=LU(:,:,t));
    timeinuse(isameuse)=timeinuse(isameuse)+1;
    timeinuse(inewuse)=1;
    
    %%%%%%%%%%%%%%%%   Agents' Expectation Models   %%%%%%%%%%%%%%%%%%%%%%%
    
    %%% YIELD EXPECTATIONS
    for iuse=1:Nuse-Nuse_nonprod
        subprodproj=prodproj(:,:,:,iuse);
        subproderror=proderror(:,:,:,iuse);
        for icol=1:NWIDTH
            for irow=1:NLENGTH
                iprodclass1=find(prodmodel(irow,icol,:)==1);
                iprodclass2=find(prodmodel(irow,icol,:)==2);
                iprodclass3=find(prodmodel(irow,icol,:)==3);
                iprodclass4=find(prodmodel(irow,icol,:)==4);
                iprodclass5=find(prodmodel(irow,icol,:)==5);
                iprodclass6=find(prodmodel(irow,icol,:)==6);
                
                subproderror(irow,icol,:) = (1-DELTA)*subproderror(irow,icol,:)+...
                    DELTA*abs(PROD(irow,icol,iuse,t)-subprodproj(irow,icol,:));
                [prodbest iprodbest] = min(subproderror(irow,icol,:),[],3);
                prodbestSAVE(irow,icol,iuse,t) = prodbest;
                iprodbestSAVE(irow,icol,iuse,t) = iprodbest;
                prodprojSAVE(irow,icol,iuse,t) = subprodproj(irow,icol,iprodbest);
                prodmodelSAVE(irow,icol,iuse,t) = prodmodel(irow,icol,iprodbest);
                
                for i = 1:PRODCLASS
                    if i == 1
                        % mimic models
                        subprodproj(irow,icol,iprodclass1) = PROD(irow,icol,iuse,t)+(1-aa...
                            (irow,icol,iprodclass1)).*(0.5*PROD(irow,icol,iuse)-...
                            (PROD(irow,icol,iuse,t)-PROD(irow,icol,iuse,t-1)));
                    elseif i == 2
                        % mean model
                        for jk = 1:length(iprodclass2)
                            subprodproj(irow,icol,iprodclass2(jk)) = mean(PROD(irow,icol,iuse,...
                                t:-1:(t-aa(irow,icol,iprodclass2(jk)))));
                        end
                    elseif i == 3
                        %cycle model
                        subprodproj(irow,icol,iprodclass3) = PROD(irow,icol,iuse,...
                            t-round(max(1,aa(irow,icol,iprodclass3))));
                    elseif i == 4
                        % projection model
                        for jl = 1:length(iprodclass4)
                            indata=zeros(MAXMEANMODEL,1);
                            %Nonlinear Forecast
                            timespan=length(t-(1+aa(irow,icol,iprodclass4(jl))):t);
                            indata(1:timespan)=PROD(irow,icol,iuse,t-(1+aa(irow,icol,iprodclass4(jl))):t);
                            pcoef=polyfit(1:timespan,indata(1:timespan)',1);
                            pline=pcoef(1).*(1:timespan+1)+pcoef(2);
                            subprodproj(irow,icol,iprodclass4(jl))=pline(length(pline));
                        end
                    elseif i == 5
                        % rescale model
                        subprodproj(irow,icol,iprodclass5) = aa(irow,icol,...
                            iprodclass5)*PROD(irow,icol,iuse,t);
                    elseif i == 6
                        irownei=max(min(irow-Neihood:irow+Neihood,NLENGTH),1);
                        icolnei=max(min(icol-Neihood:icol+Neihood,NLENGTH),1);
                        for jm=1:length(iprodclass6)
                            subprodproj(irow,icol,iprodclass6(jm)) = mean(mean(mean(PROD(irownei,icolnei,...
                                iuse,t:-1:(t-aa(irow,icol,iprodclass6(jm)))),4)));
                        end
                    end
                end
                
                EXPTPROD(irow,icol,iuse,t+1)=subprodproj(irow,icol,iprodbest);
            end
        end
        prodproj(:,:,:,iuse)=subprodproj;
        proderror(:,:,:,iuse)=subproderror;
             
        %%%% PRICE EXPECTATIONS
        subpriceproj=priceproj(:,:,iuse);
        subpriceerror=priceerror(:,:,iuse);
        for na=1:length(iNagents)
            ipriceclass1=find(pricemodel(iNagents(na),:)==1);
            ipriceclass2=find(pricemodel(iNagents(na),:)==2);
            ipriceclass3=find(pricemodel(iNagents(na),:)==3);
            ipriceclass4=find(pricemodel(iNagents(na),:)==4);
            ipriceclass5=find(pricemodel(iNagents(na),:)==5);
            ipriceclass6=find(pricemodel(iNagents(na),:)==6);
            
             subpriceerror(iNagents(na),:) = (1-DELTA)*subpriceerror(iNagents(na),:)+...
                DELTA*abs(Price(iuse,t)-subpriceproj(iNagents(na),:));
            [pricebest ipricebest] = min(subpriceerror(iNagents(na),:),[],2);
            pricebestSAVE(iNagents(na),iuse,t) = pricebest;
            ipricebestSAVE(iNagents(na),iuse,t) = ipricebest;
            priceprojSAVE(iNagents(na),iuse,t) = subpriceproj(iNagents(na),ipricebest);
            pricemodelSAVE(iNagents(na),iuse,t) = pricemodel(iNagents(na),ipricebest);
            
            for i = 1:PRICECLASS
                if i == 1
                    % mimic models
                    subpriceproj(iNagents(na),ipriceclass1) = Price(iuse,t)+(1-bb...
                        (iNagents(na),ipriceclass1)).*(0.5*Price(iuse,t)-...
                        (Price(iuse,t)-Price(iuse,t-1)));
                elseif i == 2
                    % mean model
                    for jl = 1:length(ipriceclass2)
                        subpriceproj(iNagents(na),ipriceclass2(jl)) = mean(Price(iuse,...
                            t:-1:(t-bb(iNagents(na),ipriceclass2(jl)))));
                    end
                elseif i == 3
                    %cycle model
                    subpriceproj(iNagents(na),ipriceclass3) = Price(iuse,...
                        t-round(max(1,bb(iNagents(na),ipriceclass3))));
                elseif i == 4
                    % projection model
                    for jl = 1:length(ipriceclass4)
                        %Nonlinear Forecast
                        indata=Price(iuse,t-(1+bb(iNagents(na),ipriceclass4(jl))):t);
                        pcoef=polyfit(1:length(indata),indata,1);
                        pline=pcoef(1).*(1:length(indata)+1)+pcoef(2);
                        subpriceproj(iNagents(na),ipriceclass4(jl))=pline(length(pline));
                    end
                elseif i == 5
                    % rescale model
                    subpriceproj(iNagents(na),ipriceclass5) = bb(iNagents(na),...
                        ipriceclass5)*Price(iuse,t);
                end
            end
           
            WTAPRICE(iNagents(na),iuse,t+1)=subpriceproj(iNagents(na),ipricebest);
        end
        priceproj(:,:,iuse)=subpriceproj;
        priceerror(:,:,iuse)=subpriceerror;
    end

    %%
end

%%%%%%% Results Report %%%%%%%%%
indagents=[7; 20; 59; 98];
homelabsup=zeros(length(indagents),length(TSTART+1:TMAX));
farmlabsup=zeros(length(indagents),length(TSTART+1:TMAX));
nflabsup=zeros(length(indagents),length(TSTART+1:TMAX));
sublabsup=zeros(length(indagents),length(TSTART+1:TMAX));
mktlabsup=zeros(length(indagents),length(TSTART+1:TMAX));
substore=zeros(length(indagents),length(TSTART+1:TMAX));
incstore=zeros(length(indagents),length(TSTART+1:TMAX));
intagpct=zeros(length(indagents),length(TSTART+1:TMAX));
extagpct=zeros(length(indagents),length(TSTART+1:TMAX));
pastpct=zeros(length(indagents),length(TSTART+1:TMAX));
treepct=zeros(length(indagents),length(TSTART+1:TMAX));
falpct=zeros(length(indagents),length(TSTART+1:TMAX));
CROPFREQ=zeros(NLENGTH,NWIDTH);
CF=zeros(NLENGTH,NWIDTH);
avgCROPFREQ=zeros(NLENGTH,NWIDTH);
altCF=zeros(NLENGTH,NWIDTH);

for a=1:length(indagents)
    iparcel=find(OWN == indagents(a));
    for tpost=TSTART+1:TMAX
        subLUpost=LU(:,:,tpost);
        intagpct(a,tpost-TSTART)=length(find(subLUpost(iparcel) == 1))/length(iparcel);
        extagpct(a,tpost-TSTART)=length(find(subLUpost(iparcel) == 2))/length(iparcel);
        pastpct(a,tpost-TSTART)=length(find(subLUpost(iparcel) == 3))/length(iparcel);
        treepct(a,tpost-TSTART)=length(find(subLUpost(iparcel) == 4))/length(iparcel);
        falpct(a,tpost-TSTART)=length(find(subLUpost(iparcel) == 5))/length(iparcel);
    end
    homelabsup(a,1:length(TSTART+1:TMAX))=Labor(indagents(a),1,TSTART+1:TMAX);
    farmlabsup(a,1:length(TSTART+1:TMAX))=Labor(indagents(a),2,TSTART+1:TMAX);
    nflabsup(a,1:length(TSTART+1:TMAX))=Labor(indagents(a),3,TSTART+1:TMAX);
    sublabsup(a,1:length(TSTART+1:TMAX))=Farmlabor(indagents(a),1,TSTART+1:TMAX);
    mktlabsup(a,1:length(TSTART+1:TMAX))=Farmlabor(indagents(a),2,TSTART+1:TMAX);
    substore(a,1:length(TSTART+1:TMAX))=foodbalance(indagents(a),TSTART+1:TMAX);
    incstore(a,1:length(TSTART+1:TMAX))=STORE(indagents(a),2,TSTART+1:TMAX);
    
end
for tpost=TSTART+1:TMAX
    subLUpost=LU(:,:,tpost);
    CROPFREQ=CROPFREQ+(subLUpost == 1 | subLUpost == 2);
    if tpost >= 16
        CF=CF+(subLUpost == 1 | subLUpost == 2);
    end
end
avgCROPFREQ(CROPFREQ > 0)=CROPFREQ(CROPFREQ > 0)./length(TSTART+1:TMAX);
altCF(CF > 0)=CF(CF > 0)./length(16:TMAX);

homelabgrid=zeros(NLENGTH,NWIDTH);
farmlabgrid=zeros(NLENGTH,NWIDTH);
nflabgrid=zeros(NLENGTH,NWIDTH);
subfarmgrid=zeros(NLENGTH,NWIDTH);
mktfarmgrid=zeros(NLENGTH,NWIDTH);
CEgrid=zeros(NLENGTH,NWIDTH);
for ia=1:length(iNagents) 
    iown=(OWN==iNagents(ia));
    homelabgrid(iown)=mean(Labor(iNagents(ia),1,:),3)/Ltot(iNagents(ia));
    farmlabgrid(iown)=mean(Labor(iNagents(ia),2,:),3)/Ltot(iNagents(ia));
    nflabgrid(iown)=mean(Labor(iNagents(ia),3,:),3)/Ltot(iNagents(ia));
    subfarmgrid(iown)=mean(Farmlabor(iNagents(ia),1,:),3)/mean(Labor(iNagents(ia),2,:),3);
    mktfarmgrid(iown)=mean(Farmlabor(iNagents(ia),2,:),3)/mean(Labor(iNagents(ia),2,:),3);
    CEgrid(iown)=CE(iNagents(ia));
end

avgcultfreq=mean(avgCROPFREQ(avgCROPFREQ~=0));
stdcultfreq=std(avgCROPFREQ(avgCROPFREQ~=0));
avgaltCF=mean(altCF(altCF ~= 0));
consume=repmat(SUBREQ,1,length(TMAX-9:TMAX))+foodbalance(:,TMAX-9:TMAX);
cvconsume=std(consume,1,2)./mean(consume,2);
cvconsume_ind=length(find(cvconsume >= 0.25));
cvconsumecheck=(cvconsume_ind/Nagents < 0.1);
surplus=mean(foodbalance(:,TMAX-9:TMAX)./repmat(SUBREQ,1,length(TMAX-9:TMAX)),2);
surplus_ind=length(find(surplus > 0.2))/Nagents;
surpluscheck=(surplus_ind < 0.1);
asplevel=(inclevel(:,TMAX-9:TMAX) >= A(:,TMAX-9:TMAX));
asplevel_ind=length(find(sum(asplevel,2)/length(TMAX-9:TMAX) >= 0.9));
asplevelcheck=(asplevel_ind >= 0.9*Nagents);

realfw=zeros(Nagents,TMAX);
subfw=zeros(Nagents,6);
for tt=TSTART:TMAX
    subfw=FARMWAGE(:,:,tt);
    for ja=1:Nagents
        realfw(ja,tt)=subfw(ja,ifindexptmktlab(ja,tt));
    end
end
if mean(mean(shadowprice,2)) > mean(mean(realfw,2)) && mean(mean(shadowprice,2)) < mean(mean(foodprice,2))
    mktprod_crit=(Farmlabor(:,2,TMAX-9:TMAX) > 0);
    mktprod_ind=length(find(mktprod_crit==1))/(Nagents*length(TMAX-9:TMAX));
    mktprodcheck=(mktprod_ind < 0.25);
elseif mean(mean(shadowprice,2)) <= mean(mean(realfw,2)) && mean(mean(shadowprice,2)) <= mean(mean(foodprice,2))
    mktprod_crit=(Farmlabor(:,2,TMAX-9:TMAX) > 0);
    mktprod_ind=length(find(mktprod_crit==1))/(Nagents*length(TMAX-9:TMAX));    
    mktprodcheck=(mktprod_ind >= mean(CE));
elseif mean(mean(shadowprice,2)) < mean(mean(realfw,2)) && mean(mean(shadowprice,2)) > mean(mean(foodprice,2))
    mktprod_crit=(Farmlabor(:,2,TMAX-9:TMAX) > 0);
    mktprod_ind=length(find(mktprod_crit==1))/(Nagents*length(TMAX-9:TMAX));
    mktprodcheck=(mktprod_crit > 0.75);
end

ndate=datestr(date,'ddmmyy');
fname=sprintf('results_LUC_ABM_17vbatch_base_mi20_16_%s_%d',ndate,mrun);

save(fname,'homelabsup','farmlabsup','nflabsup','sublabsup','mktlabsup',...
    'substore','incstore','intagpct','extagpct','pastpct','treepct','falpct',...
    'avgcultfreq','stdcultfreq','avgaltCF','homelabgrid','farmlabgrid','nflabgrid','subfarmgrid',...
    'mktfarmgrid','CEgrid','CE','CEra','CEneut','Price','PROD','BASEPROD',...
    'LU','STORE','Labor','inclevel','cvconsume','surplus','asplevel_ind','mktprod_ind',...
    'laborprefs','Farmlabor','SUBLABOR','SUBREQ','INCREQ','MARGPROD','A',...
    'foodbalance','cvconsume','surplus')